module vector2d;


import util.all;
import std.stdio;
import std.file;
import std.path;
import sdl.all;
import std.conv;
import demo.az01;


class InvalidCommandLine : object.Exception
{
	this(char[] msg)
	{
		super(msg);	
	}	
}


int main(char[][] args)
{	
	chdir(getDirName(args[0]));
	
	
	int fsaa = 1;	
		
	auto width = SDLApp.AUTO_DETECT;
	auto height = SDLApp.AUTO_DETECT;
	char[] sratio = "16/9"; // because black rectangles are nice
	bool fullscreen = true; // by default, fullscreen
	bool music = true; //true;	// by default, music is ON

	// parse command-line arguments
	{
		int i = 1; 
		
		while (i < args.length)
		{
			switch(args[i])
			{
				case "-width": 
				case "-w": 
					if (i + 1 == args.length) throw new InvalidCommandLine("Missing width");
					width = cast(int)atoi(args[i+1]);
					i += 2;
					break;
					
				case "-height":
				case "-h": 
					if (i + 1 == args.length) throw new InvalidCommandLine("Missing height");
					height = cast(int)atoi(args[i+1]);
					i += 2;
					break;
					
				case "-ratio":  // autorized : 16/9  4/3  16/10
				case "-r": 
					if (i + 1 == args.length) throw new InvalidCommandLine("Missing ratio");
					sratio = args[i+1];
					i += 2;
					break;
					
				case "-a":
				case "-fsaa": 
					if (i + 1 == args.length) throw new InvalidCommandLine("Missing FSAA option");
					fsaa = cast(int)atoi(args[i+1]);
					i += 2;
					break;
					
				case "-f":
				case "-fullscreen": 
					fullscreen = true;
					i += 1;
					break;
					
				case "-nf":
				case "-no-fullscreen": 
					fullscreen = false;
					i += 1;
					break;
					
				case "-nm":
				case "-no-music": 
				case "-nosound": 
				case "-no-sound": 
					music = false;
					i += 1;
					break;
						
				default: throw new InvalidCommandLine(format("Unknown argument : %s", args[i]));
			}
		}
	}
	
	if ((fsaa != 1) && (fsaa != 2) && (fsaa != 4) && (fsaa != 8) && (fsaa != 16))
		throw new InvalidCommandLine("No such FSAA option (should be 1, 2, 4, 8 or 16)");
	
	double ratio = void;	
	
	if (sratio == "4/3") ratio = 4.0/3.0;
	else if (sratio == "16/9") ratio = 16.0/9.0;
	else if (sratio == "16/10") ratio = 16.0/10.0;
	else throw new InvalidCommandLine("Use 4/3, 16/9 or 16/10 as ratio");
	
	try
	{
		// finally launch teh demo !
		auto scope app = new AZ01(width, height, ratio, fullscreen, fsaa, music);
		debug(2) writefln("run ");
		app.run();
		debug(2) writefln("run finished");
	} catch(Exception e) { }
	
	return 0;
}
