/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module audio.audiobuffer;

import audio.types;
import util.math;
import sdl.all;
import std.string;
import std.stdio;

enum SamplingMode { NEAREST = 1, 
                    LINEAR = 2 }; 
                    
enum BorderMode { REPEAT = 1, 
                  LOOP = 2 }; 	


class AudioBuffer
{
	protected
	{
		float[] data;
	}
	
	public
	{
		this(int length)
		{
			this.data.length = length;
			clear();
		}	
		
		this(float[] data)
		{
			this.data = data.dup; // duplicates sample
		}		
		
		void fill(float value)
		{
			data[] = value;
			/*for (int i = 0; i < data.length; i++)
			{
				data[i] = value;	
			}*/	
		}
		
		void clear()
		{
			fill(0.f);	
		}
		
		float * ptr()
		{
			return data.ptr;
		}
		
		bool inBuffer(int index)
		{
			return ((index >= 0)	&& (index < data.length));
		}
		
		void copy(AudioBuffer b) 
		{
			data[] = b.data[]; // in love with D :)
		}	
		
		void copy(AudioBuffer b, float factor) 
		{
			assert(b.data.length == data.length);			
			for (int i = 0; i < data.length; i++)
			{
				data[i] = b.data[i] * factor;
			}
		}	
		
		void accumulate(AudioBuffer b, float volume) 
		{
			assert(b.data.length == data.length);			
			for (int i = 0; i < data.length; i++)
			{
				data[i] = data[i] + b.data[i] * volume;
			}
		}	
		
		void accumulate(AudioBuffer b, AudioBuffer volume) 
		{
			assert(b.data.length == data.length);			
			for (int i = 0; i < data.length; i++)
			{
				data[i] = data[i] + b.data[i] * volume.data[i];
			}
		}
		
		int getIndex(int i, BorderMode bmode)
		{
			int res = i;
			if (!inBuffer(i)) 
			{
				switch(bmode)
				{
					case BorderMode.LOOP:
						
						res = i % data.length;
						if (res < 0) res += data.length;
						break;
						
					case BorderMode.REPEAT:
						if (i > data.length) 
						{
							res = data.length - 1;
						} else 
						{
							res = 0;						
						}
						break;
												
					default: res = -1;
				}
			}
			assert(res >= 0);
			assert(res < data.length);
			return res;
		}
		
		float sample(float x, SamplingMode smode, BorderMode bmode)
		{
			switch(smode)
			{
				
				case SamplingMode.NEAREST:	
				{			
					final int i = cast(int) x;	
					return data[getIndex(i, bmode)];
				}
					
				case SamplingMode.LINEAR:
				{				
					final int i_part = cast(int) floor(x);	
					final float f_part = x - i_part;
					final float d1 = data[getIndex(i_part, bmode)];
					final float d2 = data[getIndex(i_part + 1, bmode)];
					return d1 + (d2 - d1) * f_part;
				}
				
				default: ;
			}
			return float.nan;
		}
		
		
		
		void opAddAssign(AudioBuffer b) 
		{
			
			assert(b.data.length == data.length);			
			for (int i = 0; i < data.length; i++)
			{
				data[i] += b.data[i];
			}
		}	
		
		
		
		void opMulAssign(AudioBuffer b) 
		{
			assert(b.data.length == data.length);			
			for (int i = 0; i < data.length; i++)
			{
				data[i] *= b.data[i];
			}
		}
		
		void opAddAssign(float x) 
		{
			for (int i = 0; i < data.length; i++)
			{
				data[i] += x;
			}
		}
		
		void opMulAssign(float x) 
		{
			for (int i = 0; i < data.length; i++)
			{
				data[i] *= x;
			}
		}
	}
}

class CouldNotOpenSample : object.Exception
{	
	this(char[] filename)
	{
		super(format("Couldn't open %s\n", filename));
	}
}

class CouldNotConvertData : object.Exception
{	
	this(char[] msg)
	{
		super(msg);
	}
}

/*
static AudioBuffer[] readWAV(int channels = 1)
{
	Uint8 *originalBuffer;
	Uint32 origLength;
	SDL_AudioSpec origFormat;			
			
	if( SDL_LoadWAV(toStringz(filename), &origFormat, &originalBuffer, &origLength) is null )
	{
		throw new CouldNotOpenSample(filename);
	}
			
	SDL_AudioCVT cvt;
			
			
	int res = SDL_BuildAudioCVT(&cvt, 
	                            origFormat.format, 
	                            origFormat.channels, 
	                            origFormat.freq, 
	                            AUDIO_S16SYS, 
	                            channels, 
	                            SAMPLE_RATE);
	if (-1 == res)
	{
		throw new CouldNotConvertData( std.string.toString(SDL_GetError()));				
	}
			
	cvt.buf = cast(Uint8 *) malloc(origLength * cvt.len_mult);
	cvt.len = origLength;
	memcpy(cvt.buf, originalBuffer, origLength);
	SDL_FreeWAV(originalBuffer);
			
	this.buffer = cvt.buf;
	this.length = cast(Uint32) lrint(cvt.len * cvt.len_ratio);
	SDL_ConvertAudio(&cvt);	
			
}
*/