/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module audio.modules.triplefollower;

private import audio.allwhatmodulesneed;

// a module who track the volume enveloppe of three sub-bands 
// to use in demo effects
// currently : low band

final class TripleFollower : AudioModule
{	
	private
	{
		float m_env; // current enveloppe value
		float m_normalizedEnv; // current enveloppe value, normalized in regard with min and max
		float m_min;
		float m_max;
		
		
		float m_attack;
		float m_release;
		float m_ga;
		float m_gr;
		
		// lowpass filter it
		float W0, K1, K2, B0, B1, B2, A1, A2;
		float X_m_2, X_m_1, Y_m_2, Y_m_1;
		
		float X_m_22, X_m_12, Y_m_22, Y_m_12; // another -12dB/octave
		
		
	}
		
	protected
	{	
		override void doProcess()
		{
			auto inBuffers = cast(StereoAnalogData)(input(0).data);
			auto outBuffers = cast(StereoAnalogData)(output(0).data);
			
			float * inLeft = inBuffers.left.ptr;
			float * inRight = inBuffers.right.ptr;
			float * outLeft = outBuffers.left.ptr;
			float * outRight = outBuffers.right.ptr;
			
			float env = m_env;
			float ga = m_ga, gr = m_gr;
			
			for (int i = 0; i < BUFFER_SIZE; ++i)
			{
				// compute enveloppe
				
				float X = inLeft[i] + inRight[i];	
				
				float Y = B0 * X + B1 * X_m_1 + B2 * X_m_2 + A1 * Y_m_1 + A2 * Y_m_2;
				X_m_2 = X_m_1;
				Y_m_2 = Y_m_1;
				X_m_1 = X;
				Y_m_1 = Y;				
					
				float Y2 = B0 * Y + B1 * X_m_12 + B2 * X_m_22 + A1 * Y_m_12 + A2 * Y_m_22;
				X_m_22 = X_m_12;
				Y_m_22 = Y_m_12;
				X_m_12 = Y;
				Y_m_12 = Y2;
				
				
				Y = abs(Y2);
        					
				float g = (env < Y ? ga : gr);
				env = env * g + Y * (1.f - g);
				
				
				// copy to output
				outLeft[i] = inLeft[i];
				outRight[i] = inRight[i];
			}
			
			
			m_env = env;			
			
			if (env < m_min) m_min = env;
			if (env > m_max) m_max = env;
			
			m_normalizedEnv = clamp((env - m_min) / (m_max - m_min));
			m_normalizedEnv = smoothStep(0.f, 1.f, m_normalizedEnv);
			
		}	
	}
		
	public
	{	
		this() 
		{
			super("TRIPLEFOLLOWER");			
			addOutput("OUT", AudioDataType.STEREO_ANALOG); // output : copy of the input
			
			addInput("IN", AudioDataType.STEREO_ANALOG); // input
			m_env = 0.f;
			
			m_attack = 0.010; // temps de montee en s
			m_release = 0.200; // temps de descente en s
			m_ga = exp(-1/(SAMPLE_RATE_F * m_attack));
			m_gr = exp(-1/(SAMPLE_RATE_F * m_release));
			
			auto const CUTOFF = 100.f; // hz
			W0 = tan(PI_F * CUTOFF / SAMPLE_RATE_F);
			K1 = sqrt(2.f) * W0;
			K2 = W0 * W0;
			B0 = K2 / (1.f + K1 + K2);
			B1 = 2.f * B0;
			B2 = B0;
			A1 = B1 * (1.f / K2 - 1.f);
			A2 = 1.f - (B0 + B1 + B2 + A1);	
			
			X_m_2 = X_m_1 = Y_m_2 = Y_m_1 = 0.f;
			X_m_22 = X_m_12 = Y_m_22 = Y_m_12 = 0.f;
			m_min = 10.0f;
			m_max = -10.0f;
		}
		
		float env()
		{
			//return m_env;	
			return m_normalizedEnv;
		}
	}
	
}
