/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module audio.sdlrenderer;

import sdl.all;
import std.stdio;

import audio.all;
import util.math;

extern(C)
{
	private void sdlaudio_callback(void *userdata, Uint8 *stream, int len)
	{
		SDLRenderer renderer = cast(SDLRenderer) userdata;		
		renderer.process(stream, len);
	}
}

class SDLRenderer
{
	private
	{		
		SDL_AudioSpec mixerAudioFormat;
		AudioModule m_source;
		bool initialized;
		int currentCycle;
		int currentBuffer;
		SDL m_sdl; 
		
		void process(Uint8 *stream, int len)
		{	
			debug(1) std.stdio.writefln(">rendering buffer #%d", currentBuffer);	
			assert(mixerAudioFormat.channels == 2);
			assert(mixerAudioFormat.format == AUDIO_S16LSB);
			AudioModule source = m_source;
			
			auto pin = (source.output.data);
			auto buffers = cast(StereoAnalogData)(pin);
			assert(buffers !is null);
			
			int requiredSamples = len >> 2; // 2 bytes per samples, stereo
			int requiredBuffers = requiredSamples / BUFFER_SIZE;		
			
			Sint16 * dest = cast(Sint16 *) stream;
			for (int i = 0; i < requiredBuffers; i++)
			{
				source.updateTo(currentCycle);	
				currentCycle++;
				float * L = buffers.left.ptr;
				float * R = buffers.right.ptr;
				
				for (int j = 0; j < BUFFER_SIZE; j++)
				{
					final float left = L[j];
					final float right = R[j];
					*dest++ = cast(short) lrint(short.max * audioClip(left));
					*dest++ = cast(short) lrint(short.max * audioClip(right));
				}
			}	
			currentBuffer++;
			
			debug(1) std.stdio.writefln("<rendering buffer #%d", currentBuffer);	
		}		
	}
	
	public
	{
	
		this(AudioModule source)
		{
			this.initialized = false;			
			
			SDL_AudioSpec desired;
			
			desired.freq = SAMPLE_RATE;
			desired.format = AUDIO_S16SYS;
			desired.channels = 2;
			desired.samples = 1024;	
			desired.callback = &sdlaudio_callback;
			desired.userdata = cast(void *)this;
			
			SDL.instance.subSystemInit(SDL_INIT_AUDIO);
			
			if ( SDL_OpenAudio(&desired, &mixerAudioFormat) < 0 )
			{
  				writefln("Couldn't open audio: %s\n", std.string.toString(SDL_GetError()));
  				return;
			}

			this.initialized = true;
			this.m_source = source;
			this.currentCycle = 0;
			this.currentBuffer = 0;
			m_sdl = SDL.instance;
		}		
		
		~this()
		{		
			debug(2)writefln(">deleting SDLRrenderer");

			//SDL_PauseAudio(1);
			
			if (initialized)
			{				
				SDL_CloseAudio();		
				initialized = false;
			}
			
			debug(2)writefln("<deleting SDLRrenderer");
		}
		
		SDL_AudioSpec * getFormat()
		{
			return &mixerAudioFormat;
		}	
		
		AudioModule source()
		{
			return m_source;	
		}	
		
		AudioModule source(AudioModule s)
		{
			return m_source = s;	
		}
		
		void play()
		{
			SDL_PauseAudio(0);
		}
		
		void stop()
		{
			SDL_PauseAudio(1);
		}
		
		double playTime()
		{
			return currentCycle	* TIME_PER_CYCLE_D;
		}
		
	}
	
}