/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module demo.demo;

import util.all;
import util.geometry.all;
import demo.effect;
import demo.bitmapfont;
import gl.all;
import audio.all;
import sdl.all;

/**
 * A container for a list of effects and post-processing
 * provides also some info to effects
 */
class Demo
{
	private
	{
		const BLOOM_DOWNSAMPLING = 2;
		
		Vector!(Effect) m_effects;
		bool m_finished; // whether the demo is terminated 
		bool m_audioIsPlaying;
		
		int m_width;
		int m_height;
		float m_ratio;
		
		char[] m_musicFile;
		bool m_musicActivated;
		
		vec4i m_viewport;
		OGGFile m_oggFile;
		BitmapFont m_bitmapFont; // to be used by all effects
		
		/* loading stuff */
		
			Texture2D m_loadingTexture;
		
		
		/* post processing stuff */
			Texture2D m_mainTexture; // where effect are rendered, to be passed to effects
			FBO m_mainFBO; // a FBO which render to m_mainTexture, to be passed to effects
			
			Texture2D m_toneMappedTexture; // where tonemapping is applied
			ShaderProgram m_toneMappingShader;
			FBO m_toneMappingFBO; 
			
			Texture2D m_bloomTexture; // a bloom Texture 
			FBO m_bloomFBO; 
			ShaderProgram m_bloomShader;
			
			Texture2D m_bloom2Texture; // a second bloom Texture (because filter is gaussian)
			FBO m_bloom2FBO; 
			ShaderProgram m_bloom2Shader;
			
			Texture2D m_paperTexture; // a paper texture to paer render			
			
			ShaderProgram m_finalShader;
		
		SDLRenderer m_audioRenderer = null;
		OGGPlayer m_oggPlayer;
		TripleFollower enveloppeFollower;
		SDL m_SDL;
			
		
		double m_startTime; // time when the music started
		
		float m_loadingProgression; // 1.f if the loading is finished
		
		void load()
		{
			drawLoadingScreen(0.f);
			//m_bitmapFont = new BitmapFont("data/tex/font1.png", "data/tex/font2.jpg");			
			m_bitmapFont = new BitmapFont("data/tex/robin_bold.png", "data/tex/font2.jpg");			
			
			for (int i = 0; i < m_effects.size(); ++i)
			{
						
				m_loadingProgression = i / cast(float)(m_effects.size());
				drawLoadingScreen(m_loadingProgression);
				Effect e = 	m_effects.get(i);
				e.load();								
			}
			m_loadingProgression = 0.99f;
			
			if (m_musicActivated)
			{
				drawLoadingScreen(m_loadingProgression);
				auto music = new OGGFile(m_musicFile);
				auto oggBuffer = new OGGBuffer(music);
				m_oggPlayer = new OGGPlayer(oggBuffer);	
				m_oggPlayer.pause();
				enveloppeFollower.input("IN").connectTo(m_oggPlayer);
				m_audioRenderer = new SDLRenderer(enveloppeFollower);
				m_audioRenderer.play(); // begin music (but ogg is paused)
			}
			
			m_loadingProgression = 1.0001f;
			drawLoadingScreen(m_loadingProgression);
			//m_startTime = 0.f;
			return 0;			
		}
		
		void drawLoadingScreen(float progression)
		{
			// update loading screen
			FBO.getDefault().use();
			ShaderProgram.getDefault().use();
			glClearColor(0.f,0.f,0.f,1.f);
			GL.clear(GL.Buffer.ALL);												
			
			glViewport(m_viewport.x, m_viewport.y, m_viewport.z, m_viewport.w);
			
			GL.projectionMatrix = mat4f.scale( vec3f(1.f / ratio, 1.f, 1.f));
			GL.modelviewMatrix = mat4f.IDENTITY;
			
			auto xmin = -ratio * 0.8f;
			auto xmax = +ratio * 0.8f;
			auto ymin = -0.05f;
			auto ymax = +0.05f;
			auto disp = 0.015f;
			
			GL.enable([GL_TEXTURE_2D]);		
			GL.disable([GL_BLEND]);	
			m_loadingTexture.use(0);
			glColor4f(1.0f,1.0f,1.0f, 1.f);
			glBegin(GL_QUADS);
				glTexCoord2f(0,1); glVertex2f(-4.f / 3.f, -1.f);
				glTexCoord2f(1,1); glVertex2f(+4.f / 3.f, -1.f);
				glTexCoord2f(1,0); glVertex2f(+4.f / 3.f, +1.f);
				glTexCoord2f(0,0); glVertex2f(-4.f / 3.f, +1.f);
			glEnd();
			
			GL.disable([GL_TEXTURE_2D]);			
			GL.enable([GL_BLEND]);	
			GL.setBlend(GL.BlendMode.ADD, GL.BlendFactor.SRC_ALPHA, GL.BlendFactor.ONE_MINUS_SRC_ALPHA,
 			            GL.BlendMode.MAX, GL.BlendFactor.ONE, GL.BlendFactor.ONE);
			glColor4f(1.f,1.f,1.f, 0.8f);			
			glBegin(GL_QUAD_STRIP);				
				glVertex2f(xmin-disp,ymin-disp);
				glVertex2f(xmin-disp*2,ymin-disp*2);
				glVertex2f(xmin-disp,ymax+disp);
				glVertex2f(xmin-disp*2,ymax+disp*2);				
				glVertex2f(xmax+disp,ymax+disp);
				glVertex2f(xmax+disp*2,ymax+disp*2);
				glVertex2f(xmax+disp,ymin-disp);
				glVertex2f(xmax+disp*2,ymin-disp*2);
				glVertex2f(xmin-disp,ymin-disp);
				glVertex2f(xmin-disp*2,ymin-disp*2);
			glEnd();
			
			auto x = xmin + (xmax - xmin) * progression;
			glColor4f(1.f,1.f,1.f, 0.8f);
			glBegin(GL_QUADS);
				glVertex2f(xmin, ymin);
				glVertex2f(xmin, ymax);
				glVertex2f(x, ymax);
				glVertex2f(x, ymin);
			glEnd();
				
			SDL_GL_SwapBuffers();				
		}
		
	}
	
	public
	{
		this( vec4i viewport, char[] musicFile )
		{
			m_viewport = viewport;
			
			m_width = viewport.z;
			m_height = viewport.w;
			m_ratio = m_width / cast(float)m_height;
			
			m_effects = new Vector!(Effect)();
			
			// loading loading texture...
			{
				m_loadingTexture = new Texture2D(Texture.IFormat.RGBA8);
				m_loadingTexture.minFilter = Texture.Filter.LINEAR_MIPMAP_LINEAR;
				m_loadingTexture.magFilter = Texture.Filter.LINEAR;
				scope DevilImage img = new DevilImage("data/tex/loading.jpg");
				m_loadingTexture.setImage(0, img.width, img.height, Texture.Format.RGBA, Texture.Type.UBYTE, img.data, true);
			}
			
			// loading paper texture
			{
				m_paperTexture = new Texture2D(Texture.IFormat.RGBA8);
				m_paperTexture.minFilter = Texture.Filter.NEAREST;
				m_paperTexture.magFilter = Texture.Filter.NEAREST;
				m_paperTexture.wrapS = Texture.Wrap.REPEAT;
				m_paperTexture.wrapT = Texture.Wrap.REPEAT;
				scope DevilImage img = new DevilImage("data/tex/paper.jpg");
				m_paperTexture.setImage(0, img.width, img.height, Texture.Format.RGBA, Texture.Type.UBYTE, img.data, false);				
			}
			
			m_mainTexture = new Texture2D(Texture.IFormat.RGBA16F); // float to allow HDR
			m_mainTexture.minFilter = Texture.Filter.NEAREST;
			m_mainTexture.magFilter = Texture.Filter.NEAREST;
			m_mainTexture.setImage(0, width, height, Texture.Format.RGBA, Texture.Type.UBYTE, null, false);
			
			m_mainFBO = new FBO();
			m_mainFBO.color[0].setTarget(m_mainTexture, 0);
			m_mainFBO.setDrawBuffers([0]);
			m_mainFBO.setWrite(FBO.Component.COLORS);
			
			m_toneMappedTexture = new Texture2D(Texture.IFormat.RGBA8); // tone-mapped
			m_toneMappedTexture.setImage(0, width, height, Texture.Format.RGBA, Texture.Type.UBYTE, null, false);
			m_toneMappedTexture.minFilter = Texture.Filter.LINEAR;
			m_toneMappedTexture.magFilter = Texture.Filter.LINEAR; 
			
			m_toneMappingFBO = new FBO();
			m_toneMappingFBO.color[0].setTarget(m_toneMappedTexture, 0);
			m_toneMappingFBO.setDrawBuffers([0]);
			m_toneMappingFBO.setWrite(FBO.Component.COLORS);
			
			m_toneMappingShader = new ShaderProgram("data/shaders/tonemapping_vertex.glsl", "data/shaders/tonemapping_pixel.glsl");
			m_toneMappingShader.setSampler("tex", m_mainTexture, 0);
			
			m_bloomTexture = new Texture2D(Texture.IFormat.RGBA8); // blurred Vertically
			m_bloomTexture.setImage(0, width >> BLOOM_DOWNSAMPLING, height >> BLOOM_DOWNSAMPLING, Texture.Format.RGBA, Texture.Type.UBYTE, null, false);
			m_bloomTexture.minFilter = Texture.Filter.LINEAR;
			m_bloomTexture.magFilter = Texture.Filter.LINEAR;
			
			m_bloomShader = new ShaderProgram("data/shaders/bloom_vertex.glsl", "data/shaders/bloom_pixel.glsl");
			m_bloomShader.setSampler("tex", m_toneMappedTexture, 0);
			m_bloomShader.set1f("invTexHeight", (1.f / (1 << BLOOM_DOWNSAMPLING)) / cast(float)m_toneMappedTexture.height);
			
			m_bloomFBO = new FBO();
			m_bloomFBO.color[0].setTarget(m_bloomTexture, 0);
			m_bloomFBO.setDrawBuffers([0]);
			m_bloomFBO.setWrite(FBO.Component.COLORS);
			
				
			m_bloom2Texture = new Texture2D(Texture.IFormat.RGBA8);
			m_bloom2Texture.setImage(0, width >> BLOOM_DOWNSAMPLING, height >> BLOOM_DOWNSAMPLING, Texture.Format.RGBA, Texture.Type.UBYTE, null, false);
			m_bloom2Texture.minFilter = Texture.Filter.LINEAR;
			m_bloom2Texture.magFilter = Texture.Filter.LINEAR;
			
			m_bloom2Shader = new ShaderProgram("data/shaders/bloom2_vertex.glsl", "data/shaders/bloom2_pixel.glsl");
			m_bloom2Shader.setSampler("tex", m_bloomTexture, 0);
			m_bloom2Shader.set1f("invTexWidth", 1.f / m_bloomTexture.width);
		
			m_bloom2FBO = new FBO();
			m_bloom2FBO.color[0].setTarget(m_bloom2Texture, 0);
			m_bloom2FBO.setDrawBuffers([0]);
			m_bloom2FBO.setWrite(FBO.Component.COLORS);
	
			m_finalShader = new ShaderProgram("data/shaders/final_vertex.glsl", "data/shaders/final_pixel.glsl");
			m_finalShader.setSampler("tonemappedTexture", m_toneMappedTexture, 0);
			m_finalShader.setSampler("bloomTexture", m_bloom2Texture, 1);
			m_finalShader.setSampler("paperTexture", m_paperTexture, 2);
			m_finalShader.set1f("paperInvSize", 1.f / m_paperTexture.width);
			
			m_musicFile = musicFile;
			
			m_audioIsPlaying = false;
			m_finished = false;
			
			m_loadingProgression = 0.0f;
			m_bitmapFont = null;
			
			m_musicActivated = (musicFile != "");
			enveloppeFollower = new TripleFollower();
			
			m_SDL = SDL.instance;
		}	
		
		~this()
		{
			debug(2) writefln(">deleting demo");	
			
			//m_audioRenderer.stop();
		}
		
		void loadEverything()
		{
			if (m_loadingProgression < 1.f)
			{
				load();				
			}			
		}
		
		void start(double globalTime)
		{	
			if (!m_audioIsPlaying)
			{	
				if (m_musicActivated) m_oggPlayer.play(); // begin ogg playback
				m_startTime = globalTime;
				m_audioIsPlaying = true;
			}
		}	
		
		final Texture2D mainTexture()
		{
			return m_mainTexture;
		}
		
		final FBO mainFBO()
		{
			return m_mainFBO;
		}
		
		final int width()
		{
			return m_width;
		}
		
		final int height()
		{
			return m_height;
		}
		
		final float ratio()
		{
			return m_ratio;	
		}
		
		final float musicVolume()
		{
			return enveloppeFollower.env();
		}
		
		final BitmapFont bitmapFont()
		{
			return m_bitmapFont;
		}
		
		/// add an Effect to the demo
		void add(Effect e)
		{
			m_effects.add(e);	
			e.demo = this;
		}	
		
		void draw(double globalTime)
		{
			if (m_loadingProgression >= 1.f)
			{
				double correctedTime = globalTime - m_startTime; // to correct with the time the music took to go
				
				auto const MAX_CONCURRENT_EFFECTS = 16;
				Effect[MAX_CONCURRENT_EFFECTS] activeEffects;
				int nActiveEffects = 0;
				
				foreach (Effect e; m_effects)
				{
					if (e.isActive(correctedTime))
					{
						activeEffects[nActiveEffects++] = e;
					}
				}
				
				
				glViewport(0, 0, width, height);	
				
				bool paperRendering = false;
				
				if (nActiveEffects > 0)
				{
					
					// sort active effects to draw with the right order
					// we use a stupid bubble sort since there is often few effect at the same time
					for (int i = 0; i < nActiveEffects - 1; ++i)
					{
						for (int j = i+1; j < nActiveEffects; ++j)
						{	
							if (activeEffects[j] < activeEffects[i])
							{
								auto tmp = activeEffects[j];
								activeEffects[j] = activeEffects[i];
								activeEffects[i] = tmp;
							}
						}
					}
					
					// draw effects in order
					for (int i = 0; i < nActiveEffects; ++i)
					{
						debug(1) writefln("Drawing %s", activeEffects[i].name);
						if (activeEffects[i].isPaperEffect()) 
						{
							paperRendering = true;
						}
						activeEffects[i].draw(correctedTime);
					}					
				} 
				
				/////////////////////
				// post-processing //
				/////////////////////
				
				GL.projectionMatrix = mat4f.IDENTITY;
				GL.modelviewMatrix = mat4f.IDENTITY;
		
				// PASS 1
				// add thin black borders transparent borders up and down
				// NOT NICE
			/*	
				GL.disable( [ GL_TEXTURE_2D, GL_DEPTH_TEST, GL_ALPHA_TEST, GL_LINE_SMOOTH, GL_POLYGON_SMOOTH, GL_POINT_SMOOTH] );
				GL.enable(GL_BLEND);	
				mainFBO().use();
				ShaderProgram.getDefault().use();
			
				GL.setBlend(GL.BlendMode.ADD, GL.BlendFactor.SRC_ALPHA, GL.BlendFactor.ONE_MINUS_SRC_ALPHA,
 			            GL.BlendMode.MAX, GL.BlendFactor.ONE, GL.BlendFactor.ONE);
 			             			            
 			    glBegin(GL_QUADS);
 			    	glColor4f(0.0,0.0,0.0,1.0);	
 			    	glVertex2f(-1,-1);	
					glVertex2f(+1,-1);
					glColor4f(0.0,0.0,0.0,0.0);	
					glVertex2f(+1,-0.9);
					glVertex2f(-1,-0.9);
 			    	glVertex2f(-1,+0.9);	
					glVertex2f(+1,+0.9);
					glColor4f(0.0,0.0,0.0,1.0);	
					glVertex2f(+1,+1.0);
					glVertex2f(-1,+1.0);				
				glEnd();
				*/
				
				// PASS 2
				// tone-mapping
				
				GL.enable([GL_TEXTURE_2D]);			
				GL.disable( [ GL_DEPTH_TEST, GL_ALPHA_TEST, GL_LINE_SMOOTH, GL_POLYGON_SMOOTH, GL_POINT_SMOOTH, GL_BLEND ] );			
				m_toneMappingFBO.use();			
				m_toneMappingShader.use();
				glViewport(0, 0, width, height);
				glBegin(GL_QUADS);
					glTexCoord2f(0,0); glVertex2f(-1,-1);	
					glTexCoord2f(1,0); glVertex2f(+1,-1);
					glTexCoord2f(1,1); glVertex2f(+1,+1);
					glTexCoord2f(0,1); glVertex2f(-1,+1);				
				glEnd();
				
				// PASS 3
				// bloom first pass			
				m_bloomFBO.use();			
				m_bloomShader.use();
				glViewport(0, 0, width >> BLOOM_DOWNSAMPLING, height >> BLOOM_DOWNSAMPLING);
				glBegin(GL_QUADS);
					glTexCoord2f(0,0); glVertex2f(-1,-1);	
					glTexCoord2f(1,0); glVertex2f(+1,-1);
					glTexCoord2f(1,1); glVertex2f(+1,+1);
					glTexCoord2f(0,1); glVertex2f(-1,+1);				
				glEnd();
				
				// PASS 4
				// bloom second pass : now we have an image concolved with a 7x7 gaussian filter			
				m_bloom2FBO.use();			
				m_bloom2Shader.use();
				glViewport(0, 0, width >> BLOOM_DOWNSAMPLING, height >> BLOOM_DOWNSAMPLING);
				glBegin(GL_QUADS);
					glTexCoord2f(0,0); glVertex2f(-1,-1);	
					glTexCoord2f(1,0); glVertex2f(+1,-1);
					glTexCoord2f(1,1); glVertex2f(+1,+1);
					glTexCoord2f(0,1); glVertex2f(-1,+1);				
				glEnd();
				
				// PASS 5 : final render (bloom + main)
				FBO.getDefault().use();
				m_finalShader.use();
				m_finalShader.set1i("paper", paperRendering ? 1 : 0);
				
								
				glViewport(m_viewport.x, m_viewport.y, m_viewport.z, m_viewport.w);			
				
				glBegin(GL_QUADS);
					glTexCoord2f(0,0); glVertex2f(-1,-1);	
					glTexCoord2f(1,0); glVertex2f(+1,-1);
					glTexCoord2f(1,1); glVertex2f(+1,+1);
					glTexCoord2f(0,1); glVertex2f(-1,+1);				
				glEnd();
			}
		}		
		
		void move(double globalTime, double dt)
		{
			if (m_loadingProgression >= 1.f)
			{
				start(globalTime);	// take the start time
				
				double correctedTime = globalTime - m_startTime; // to correct with the time the music took to go
				bool res = false;
				bool finished = true;
				foreach (Effect e; m_effects)
				{
					if (e.isActive(correctedTime))
					{
						e.move(correctedTime, dt);	
						res = true;
					} 
					if (e.endTime > correctedTime) finished = false;
				}
				
				if (finished)
				{
					m_audioRenderer.stop();	
				}
				m_finished = finished;				
			} else
			{				
				SDL_Delay(20);	
			}
		}
		
		bool finished()
		{
			return m_finished;
		}
		
		float BPM()
		{
			return 110.f;
		}
		
		float timePerBeat()
		{
			return 60.f / BPM;
		}
		
		void timeTravel(float dt)
		{
			m_startTime -= dt;	
			m_oggPlayer.timeTravel(dt);
		}
		
		
		// utility : load a texture in one line
   		static Texture2D loadTexture2D(char[] filename, Texture.Filter minfilter, Texture.Filter magfilter, Texture.Wrap wrap, bool mipmap)
   		{
	   		Texture2D res = new Texture2D(Texture.IFormat.RGBA8);
			res.minFilter = minfilter;
			res.magFilter = magfilter;
			res.wrapS = wrap;
			res.wrapT = wrap;
			scope DevilImage img = new DevilImage(filename);
			res.setImage(0, img.width, img.height, Texture.Format.RGBA, Texture.Type.UBYTE, img.data, mipmap);
			return res;
   		}   		
	}
}

