/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module demo.orangeeffect;


private import demo.effect;
private import demo.demo;
private import util.all;
private import util.geometry.all;
private import gl.all;


final class OrangeEffect : Effect
{
	private 
	{
		Ball[] m_ball;		
		
		ShaderProgram m_shader;
		Texture2D m_hardtex;
		Texture2D m_graintex;
		const int NPOINTS = 5;
	}
	
	
	
	public
	{
		this(vec2d interval, int priority)
		{
			super("OrangeEffect", interval, priority);
		}		
		
		override bool isPaperEffect() { return false; }
			
		override void onLoad() // load resources
		{
			
			m_ball.length = NPOINTS;
			for (int i = 0; i < m_ball.length; ++i)
			{
				m_ball[i].pos = vec3f( randGaussf , randGaussf , 0.f  );
				m_ball[i].mov = vec3f( randGaussf, randGaussf , 0.f );
			}
			
			m_hardtex = demo.loadTexture2D("data/tex/hex_r.jpg", Texture.Filter.LINEAR, Texture.Filter.LINEAR, Texture.Wrap.REPEAT, false);
			m_graintex = demo.loadTexture2D("data/tex/orangegrain.jpg", Texture.Filter.LINEAR_MIPMAP_LINEAR, Texture.Filter.LINEAR, Texture.Wrap.REPEAT, true);
				
			m_shader = new ShaderProgram("data/shaders/orange_vertex.glsl", "data/shaders/orange_pixel.glsl");
			m_shader.setSampler("tex", m_hardtex, 0);
			m_shader.setSampler("grain", m_graintex, 1);
		}
		
		
		
		override void onDraw() // draw effect
		{
			
			double time = localTime;
			
			demo.mainFBO.use();			
				
			// define camera
			float angle = 0.f; //1.f * TWO_PI_F * smoothStep(timePerBeat * 8.f, timePerBeat * 10.f, localTime);
			float dist = 1.f * (1.f - smoothStep(timePerBeat * 12.f, timePerBeat * 16.f, localTime));
			auto globalI = step(timePerBeat * 12.f, timePerBeat * 16.f, localTime) * 100.f;
			dist += globalI * 100.f;
			auto modelView = mat4f.lookAt(vec3f(dist * sin(angle),0,-dist * cos(angle)), vec3f(0,0,0), vec3f(0.f,1.f,0.f));
		    auto projection = mat4f.perspective( 50, 1.0, 0.1, 200.0);		    
		    auto totalMatrix = modelView; //projection * modelView;
			
			vec3f[NPOINTS] pos;
			float[NPOINTS] intensity;
			
			for (int i = 0; i < m_ball.length; ++i)
			{
				auto tr = totalMatrix * vec4f(m_ball[i].pos, 1.0);
				pos[i] = tr.xyz / tr.w;		
				float I1 = (4.f + 2.f * ((localBeat + i) % NPOINTS )) / tr.w;
				float I2 = (4.f + 2.f * ((localBeat + i) % NPOINTS )) * pos[i].z;
				
				intensity[i] = lerp(I1, I2, smoothStep(timePerBeat * 4.f, timePerBeat * 8.f, localTime));
			}			
			
		    m_shader.set3fv("pos", pos);
		    m_shader.set1fv("intensity", intensity);
		    m_shader.set1f("localTime", time);
		    m_shader.set1f("env", demo.musicVolume);
		    
		    m_shader.set1f("zoom", 1.f / (0.05f + smoothStep(timePerBeat * 0.f, timePerBeat * 2.f, localTime)) + 0.2f * demo.musicVolume );
		    
		  	m_shader.use();
		  
		    
		    GL.disable( GL_DEPTH_TEST, GL_ALPHA_TEST, GL_LINE_SMOOTH, GL_POLYGON_SMOOTH, GL_POINT_SMOOTH, GL_BLEND );
		    
		    auto ratio = demo.ratio;
		    
		    GL.projectionMatrix = mat4f.scale(vec3f(1.f / demo.ratio, 1.f, 1.f)); // to span all the screen
			GL.modelviewMatrix = mat4f.IDENTITY;			
			
			glBegin(GL_QUADS);
				glTexCoord2f(-ratio,1); glVertex2f(-ratio,-1);	
				glTexCoord2f(ratio,1); glVertex2f(+ratio,-1);
				glTexCoord2f(ratio,-1); glVertex2f(+ratio,+1);
				glTexCoord2f(-ratio,-1); glVertex2f(-ratio,+1);				
			glEnd();
		}
		
		void onMove(double dt) // move effect
		{
			for (int i = 0; i < m_ball.length; ++i)
			{
				m_ball[i].move(dt);
			}	
		}
	}
	
	
}


private struct Ball
{
	public
	{
		vec3f pos;
		vec3f mov;
		vec4f color;
		float shininess;
		
		void move(float dt)
		{
			mov += (0.03f * dt) * vec3f(randGaussf(), randGaussf(), randGaussf());
			mov += (vec3f.ZEROS - pos) * 0.3f * dt;				
			pos += (mov * dt);
		}
	}
}
