/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module demo.particleset;

import util.all;
import util.geometry.all;
import gl.all;
import demo.gdologo;


// bill-boarded particle
private struct Particle
{
	public
	{
		vec3f pos; // position
		vec3f target; // the place the particle want to go
		vec3f mov; // the speed
		vec3f color; // the color
		vec3f targetcolor;
		float targetangle;
		float vangle;
		float angle;
		float nervosity;
		float attraction;
		
		void move(float dt, float viscoFactor, float attractionMod, float noise)
		{
			vec3f acc = (target - pos) * attraction * attractionMod + (nervosity+noise) * vec3f(randFloat() - 0.5f, randFloat() - 0.5f, randFloat() - 0.5f);
			mov += acc * dt;			
			mov *= viscoFactor;
			pos += mov * dt;
			
		//	color = color + (targetcolor - color) * dt * attraction * attractionMod;
			
			float accangle = (targetangle - angle) * attraction * attractionMod + 20.f * nervosity * (randFloat() - 0.5f);
			vangle += accangle * dt;
			vangle *= viscoFactor;
			angle = targetangle;
		}	
		
		void draw(mat4f transfo)
		{
			// find the billboard position
			vec4f v = transfo * vec4f(pos, 1.f);
			vec2f onscreen = v.xy / v.w; // divide by W is essential
			
			float size = 0.5f * 0.15f / v.w; // somewhat arbitrary
			
			float sina = void, cosa = void;
			sincos(angle, sina, cosa);
			
			vec2f a = onscreen + vec2f(cosa, sina) * size;
			vec2f b = onscreen + vec2f(-sina, cosa) * size;
			vec2f c = onscreen + vec2f(-cosa, -sina) * size;
			vec2f d = onscreen + vec2f(sina, -cosa) * size;
			
			
				glColor4f(color.x, color.y, color.z, 1.f); // change the color for this particle				
				glTexCoord2f(0.f, 0.f); glVertex2fv(a.ptr);
				glTexCoord2f(1.f, 0.f); glVertex2fv(b.ptr);
				glTexCoord2f(1.f, 1.f); glVertex2fv(c.ptr);
				glTexCoord2f(0.f, 1.f); glVertex2fv(d.ptr);
			
	//		glPopMatrix();
			
		}
	}
}

class ParticleSet
{
	private
	{
		Vector!(Particle) m_particles;	
		float m_viscosity; // speed attenuation
		float m_attraction; // the "speed" of the particle
		
		Texture2D m_particleTexture;
		ShaderProgram m_shader;
	}
	
	public
	{
		this(char[] imageFile)
		{
			m_particles = new Vector!(Particle)();
			auto scope di = new DevilImage(imageFile);			
			
			m_particleTexture = new Texture2D(Texture.IFormat.RGBA8);
			m_particleTexture.setImage(0, di.width, di.height, Texture.Format.RGBA, Texture.Type.UBYTE, di.data, true);
			m_particleTexture.minFilter = Texture.Filter.LINEAR_MIPMAP_LINEAR;
			m_particleTexture.magFilter = Texture.Filter.LINEAR;
			
			m_shader = new ShaderProgram("data/shaders/particle_vertex.glsl", "data/shaders/particle_pixel.glsl");
			m_shader.setSampler("tex", m_particleTexture, 0);
		}
		
		this(char[] imageFile, int n)
		{
			this(imageFile);
			for (int i = 0; i < n; ++i)
			{
				addParticle(vec3f(randGaussf(), randGaussf(), 0.f));
			}	
			
			m_attraction = 20.f;
			m_viscosity = 8.f;		
		}
		
		void addParticle(vec3f pos)
		{
			Particle p;
			p.pos = pos;
			p.target = vec3f(randGaussf(), randGaussf(), 0.f);
			p.mov = vec3f.ZEROS;	
			p.color = vec3f(0.f, 0.f, 0.f);
			p.targetcolor = vec3f(0.f, 0.f, 0.f);
			p.angle = 0;
			p.targetangle = 0;
			p.vangle = 0;
			p.nervosity = 1.f + randGaussf() * 0.5f;
			p.attraction = 1.f + randGaussf() * 0.5f;
			m_particles.add(p);
		}
		
		float viscosity() { return m_viscosity; }
		float viscosity(float v) { return m_viscosity = v; }
		
		float attraction() { return m_attraction; }
		float attraction(float a) { return m_attraction = a; }
		
		
		void move(float dt, float noise)
		{
			auto viscoFactor = exp(-dt * m_viscosity);
			
			foreach(inout Particle p; m_particles)
			{
				p.move(dt, viscoFactor, m_attraction, noise);
			}
		}
		
		void draw(mat4f cameraMatrix, float intensity, float alpha)
		{
			assert(m_particles.length > 0);
			
			GL.enable([GL_TEXTURE_2D, GL_BLEND]);			
			GL.disable( [GL_DEPTH_TEST, GL_ALPHA_TEST, GL_LINE_SMOOTH, GL_POLYGON_SMOOTH, GL_POINT_SMOOTH] );
			GL.setBlend(GL.BlendMode.ADD, GL.BlendFactor.SRC_ALPHA, GL.BlendFactor.ONE_MINUS_SRC_ALPHA,
 			            GL.BlendMode.MAX, GL.BlendFactor.ONE, GL.BlendFactor.ONE);
		    
 			            

 			m_shader.set1f("intensity", intensity);            
 			m_shader.set1f("alpha", alpha);    
			m_shader.use();		    
			
			glBegin(GL_QUADS);
			foreach(inout Particle p; m_particles)
			{
				p.draw(cameraMatrix);
			}		
			glEnd();	
			
		}
		
		// target a random form
		void nawak()
		{
			foreach(inout Particle p; m_particles)
			{
				p.targetangle = randGaussf() * PI_F;
				p.target = vec3f(randGaussf(), randGaussf(), 0.f);
				p.targetcolor = vec3f(1,1,1);
				p.color = vec3f(1,1,1);
			}
		}
		
		// target a GDO
		void target(GDOLogo logo)
		{
			assert(logo !is null);
			foreach(inout Particle p; m_particles)
			{
				vec2f target = void;
				float angle = void;
				vec4f color;
				logo.randPoint(target, angle, color);				
				
				p.targetangle = angle + randGaussf() * 0.1f;
				p.targetcolor = color.xyz;
				p.color = color.xyz;
				p.target = vec3f(target, 0.f);
			}
			/*
			auto scope lines = logo.lines();
			for (int i = 0; i < m_particles.length; ++i)
			{
				int j = i;
				while (j > lines.length()) j -= lines.length();
				GDOLogo.GDOLine g = lines[j];				
				
				vec2f target = g.getPoint(randFloat());
				float angle = g.angle();
				target.x = -target.x;
				angle = -angle;
				
				m_particles[i].targetangle = angle;
				m_particles[i].target = vec3f(target, 0.f);
			}
			*/
		}
	}
}