/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module demo.tunneleffect;

import demo.effect;
import util.all;
import util.geometry.all;
import gl.all;
import demo.gdologo;
import demo.tunnelpath;
import demo.particleset;




final class TunnelEffect : Effect
{
	private 
	{
		ShaderProgram m_shader;
		Texture2D m_tex;
		
		
		InterestingTunnelPath m_path;
		
		float m_tunnelProgression;
		mat4f m_campos;
		
		quatf m_camRot;
		
		GDOLogo[] m_logos;
		ParticleSet m_particleSet;
		GDOLogo m_current = null;
		
		
		mat4f idealCampos()
		{
			auto cpos = m_path.getPos(m_tunnelProgression);
	    	auto cup = m_path.getUp(m_tunnelProgression);
		    auto cdir = m_path.getDir(m_tunnelProgression);
		    auto cright = m_path.getRight(m_tunnelProgression);	
		    
		    

		    //cam = mat4f.rotateY(0.2f * cos(3.f * localTime)) * cam;
			//cam = mat4f.rotateX(0.2f * sin(localTime)) * cam;
		    return mat4f.lookAt(cpos - cdir, cpos, cup);
		}
		
	}
	
	
	
	public
	{
		
		this(vec2d interval, int priority)
		{
			super("TunnelEffect", interval, priority);
		}		
		
		override bool isPaperEffect() { return false; }
		
		override void onLoad() // load resources
		{	
			m_shader = new ShaderProgram("data/shaders/tunnel_vertex.glsl", "data/shaders/tunnel_pixel.glsl");	
			
			auto scope img = new DevilImage("data/tex/tex13.jpg");
			
			//auto scope img = new DevilImage("data/tex/flesh007.jpg");
			
			
			
			m_tex = new Texture2D(Texture.IFormat.RGBA8);
			m_tex.minFilter = Texture.Filter.LINEAR_MIPMAP_LINEAR;
			m_tex.magFilter = Texture.Filter.LINEAR;
			m_tex.wrapS = Texture.Wrap.REPEAT;
			m_tex.wrapT = Texture.Wrap.REPEAT;
			m_tex.setImage(0, img.width, img.height, Texture.Format.RGBA, Texture.Type.UBYTE, img.data, true);			
			
			m_shader.setSampler("tex", m_tex, 0);
			
			m_path = new InterestingTunnelPath(300, 10.f, vec3f(0,0,0), vec3f(0,1,0), vec3f(0,0,-1));
			
			m_tunnelProgression = 50.f;
			m_camRot = quatf.ONE;
			m_campos = idealCampos();
			
			static char[][] GREETZ = 
			[
				"orb.GDO", 
				"BPS.GDO",	
				"Frequency.GDO",						
				"Sector1.GDO",
				"atz.GDO",
				"blockos.GDO",
				"tits.GDO",
				"COCOON2.GDO",
 				"Mkd.GDO", 
 				"Popsy.GDO",				
				"tpolm.GDO",
				"traction.gdo", 
			];

			
			m_logos.length = GREETZ.length;
			
			// GODs
			for (int i = 0; i < m_logos.length; ++i)
			{
				m_logos[i] = new GDOLogo("data/gdo/Greetzs/" ~ GREETZ[i]);
			}
			m_particleSet = new ParticleSet("data/tex/particle2.png", 400);
			m_particleSet.nawak();
			
			
		}		
		
		
		override void onDraw() // draw effect
		{
			// passe 1 : le decor
			demo.mainFBO.use();
			
		    m_shader.use();
			    
		    GL.disable( GL_CULL_FACE, GL_LIGHTING,  GL_ALPHA_TEST, GL_LINE_SMOOTH, GL_POLYGON_SMOOTH, GL_POINT_SMOOTH );
		    GL.disable( [GL_DEPTH_TEST] );
		    GL.enable( [GL_TEXTURE_2D] );
		    GL.enable( [GL_BLEND] );
	    	GL.setBlend(GL.BlendMode.ADD, GL.BlendFactor.SRC_ALPHA, GL.BlendFactor.ONE_MINUS_SRC_ALPHA, 
		                GL.BlendMode.MAX, GL.BlendFactor.ONE, GL.BlendFactor.ONE);		    
		    
		    
		    glClear(GL_COLOR_BUFFER_BIT);
		
		    mat4f ideal = idealCampos();
	    	m_campos = ideal;
	    	
		    
	    	GL.projectionMatrix =  mat4f.scale(vec3f(1.f / demo.ratio, 1.f, 1.f)) * mat4f.perspective( 55, 1.f, 0.01, 1000.f);
			GL.modelviewMatrix = m_campos;
		    
		     for (int i = 50; i >= -20; i--)
		     {
		    	float t = m_tunnelProgression + i * 0.2f;
		    	float t2 = m_tunnelProgression + (i+1) * 0.2f;
		    	
		    	if (t < 1) continue;    	
		    	if (t2 < 1) continue;    	
		    	
		    	auto pos = m_path.getPos(t);
		    	auto up = m_path.getUp(t);
		    	auto dir = m_path.getDir(t);
		    	auto right = m_path.getRight(t);
		    	auto radius = m_path.getRadius(t);
		    	
		    	auto pos2 = m_path.getPos(t2);
		    	auto up2 = m_path.getUp(t2);
		    	auto dir2 = m_path.getDir(t2);
		    	auto right2 = m_path.getRight(t2);
		    	auto radius2 = m_path.getRadius(t2);
		 
		    	vec3f a = (up + right) * radius;
		    	vec3f b = (up - right) * radius;
		    	vec3f c = (-up - right) * radius;
		    	vec3f d = (-up + right) * radius;
		    	vec3f e = (up2 + right2) * radius2;
		    	vec3f f = (up2 - right2) * radius2;
		    	vec3f g = (-up2 - right2) * radius2;
		    	vec3f h = (-up2 + right2) * radius2;
		    	
		    	auto tunnelAlpha = 1.f; //smoothStep(0 * timePerBeat, 8 * timePerBeat, localTime);
		    	
		    	auto tunnelColor = hsla_to_rgba(localTime * 0.2f, 0.1f, 0.9, 1.f);
		    	
		    	m_shader.set4f("color", vec4f(1.f) + demo.musicVolume * vec4f(1.f,0.6,0.f,0.f));
		    	m_shader.set1f("alpha", 0.5f * tunnelAlpha);
		    	glBegin(GL_QUAD_STRIP);
		    		glColor3f(i / 50.f, 0.f, 0.f); glTexCoord2f(0,t); GL.vertex(pos + 5 * a);
		    		glColor3f((i+1) / 50.f, 0.f, 0.f); glTexCoord2f(0,t2); GL.vertex(pos2 + 5 * e);
		    		glColor3f(i / 50.f, 0.f, 0.f); glTexCoord2f(5,t); GL.vertex(pos + 5 * b);	
		    		glColor3f((i+1) / 50.f, 0.f, 0.f); glTexCoord2f(5,t2); GL.vertex(pos2 + 5 * f);
		    		glColor3f(i / 50.f, 0.f, 0.f); glTexCoord2f(0,t); GL.vertex(pos + 5 * c);
		    		glColor3f((i+1) / 50.f, 0.f, 0.f); glTexCoord2f(0,t2); GL.vertex(pos2 + 5 * g);
		    		glColor3f(i / 50.f, 0.f, 0.f); glTexCoord2f(5,t); GL.vertex(pos + 5 * d);
		    		glColor3f((i+1) / 50.f, 0.f, 0.f); glTexCoord2f(5,t2); GL.vertex(pos2 + 5 * h);
		    		glColor3f(i / 50.f, 0.f, 0.f); glTexCoord2f(0,t); GL.vertex(pos + 5 * a);		    		
		    		glColor3f((i+1) / 50.f, 0.f, 0.f); glTexCoord2f(0,t2); GL.vertex(pos2 + 5 * e);		    		
		    	glEnd();
		    
		    	
		    	m_shader.set1f("alpha", 0.5f * tunnelAlpha);
		    	glBegin(GL_QUAD_STRIP);
		    		glColor3f(i / 50.f, 0.f, 0.f); glTexCoord2f(0,t); GL.vertex(pos + 3 * a);
		    		glColor3f((i+1) / 50.f, 0.f, 0.f); glTexCoord2f(0,t2); GL.vertex(pos2 + 3 * e);
		    		glColor3f(i / 50.f, 0.f, 0.f); glTexCoord2f(5,t); GL.vertex(pos + 3 * b);	
		    		glColor3f((i+1) / 50.f, 0.f, 0.f); glTexCoord2f(5,t2); GL.vertex(pos2 + 3 * f);
		    		glColor3f(i / 50.f, 0.f, 0.f); glTexCoord2f(0,t); GL.vertex(pos + 3 * c);
		    		glColor3f((i+1) / 50.f, 0.f, 0.f); glTexCoord2f(0,t2); GL.vertex(pos2 + 3 * g);
		    		glColor3f(i / 50.f, 0.f, 0.f); glTexCoord2f(5,t); GL.vertex(pos + 3 * d);
		    		glColor3f((i+1) / 50.f, 0.f, 0.f); glTexCoord2f(5,t2); GL.vertex(pos2 + 3 * h);
		    		glColor3f(i / 50.f, 0.f, 0.f); glTexCoord2f(0,t); GL.vertex(pos + 3 * a);		    		
		    		glColor3f((i+1) / 50.f, 0.f, 0.f); glTexCoord2f(0,t2); GL.vertex(pos2 + 3 * e);		    		
		    	glEnd();
		    	
		 		m_shader.set1f("alpha", 1.5f * tunnelAlpha);
		 		
		    	
		    
		      	glBegin(GL_QUAD_STRIP);
		    		glColor3f(i / 50.f, 0.f, 0.f); glTexCoord2f(0,t); GL.vertex(pos + a);
		    		glColor3f((i+1) / 50.f, 0.f, 0.f);glTexCoord2f(0,t2); GL.vertex(pos2 + e);
		    		glColor3f(i / 50.f, 0.f, 0.f); glTexCoord2f(2,t); GL.vertex(pos + b);	
		    		glColor3f((i+1) / 50.f, 0.f, 0.f);glTexCoord2f(2,t2); GL.vertex(pos2 + f);
		    		glColor3f(i / 50.f, 0.f, 0.f); glTexCoord2f(0,t); GL.vertex(pos + c);
		    		glColor3f((i+1) / 50.f, 0.f, 0.f);glTexCoord2f(0,t2); GL.vertex(pos2 + g);
		    		glColor3f(i / 50.f, 0.f, 0.f); glTexCoord2f(2,t); GL.vertex(pos + d);
		    		glColor3f((i+1) / 50.f, 0.f, 0.f);glTexCoord2f(2,t2); GL.vertex(pos2 + h);
		    		glColor3f(i / 50.f, 0.f, 0.f); glTexCoord2f(0,t); GL.vertex(pos + a);		    		
		    		glColor3f((i+1) / 50.f, 0.f, 0.f);glTexCoord2f(0,t2); GL.vertex(pos2 + e);		    		
		    	glEnd();
	    	}
	    	
	    	// GDO rendering
	    	{
		        // for billboard
				GL.projectionMatrix = mat4f.scale(vec3f(1.f / demo.ratio, 1.f, 1.f)); // to span all the screen
				GL.modelviewMatrix = mat4f.IDENTITY;
				
				// for world to camera transformation
				auto cameraView = m_campos;
			    auto cameraProjection = mat4f.perspective( 55, 1.f, 0.01, 1000.f);
			    auto cameraMatrix = cameraProjection * cameraView;
			    
			    ;
			    
			    auto mobile = mat4f.translate(m_path.getPos(m_tunnelProgression));
			    
			    
		    	auto beat = localBeat();
			  
			   
			    if (m_current !is null)
			    { 
				    auto up = m_path.getUp(m_tunnelProgression);
		    		auto ri = m_path.getRight(m_tunnelProgression);
		    		auto corr = mat4f(-ri.x, up.x, 0.f, 0.f,
		    		                  -ri.y, up.y, 0.f, 0.f,
		    		                  -ri.z, up.z, 1.f, 0.f,
		    		                   0.f,  0.f, 0.f, 1.f);
		    		
		    		auto mat = cameraMatrix * mobile * corr;
		    		
					m_particleSet.draw(mat, 3.0f + 1.0 * demo.musicVolume(), smoothStep(28.f * timePerBeat, 32.f * timePerBeat, localTime));
			    	
			    	
					if (validation())
					{
						glPushMatrix();
						glScalef(2.0,2.0,1.f);
				    	// draw that GDO
				    	
				    	//auto color = hsla_to_rgba(localTime * 0.2f, 1.0f, 0.5f, 1.f);
				    	auto color2 = vec4f(5.0,5.0,5.0,1.0) ;
						//m_current.draw(mat, vec4f(-0.2f,-0.2f,-0.2f, 0.5f), 2.f * (0.025f + demo.musicVolume * 0.03f));
						//m_current.draw(mat, vec4f(color.xyz * 1.0f, 1.0f), 0.010 + demo.musicVolume * 0.03f, true);
				    	m_current.draw(mat, vec4f(color2.xyz * 1.f, 1.0f), 1.0f * (0.010 + demo.musicVolume * 0.03f), true);
				//    		m_current.draw(mat, vec4f(0.f,0.f,0.f, 0.3f), 0.15f * (0.010 + demo.musicVolume * 0.03f), false);
				//    		m_current.draw(mat, vec4f(0.f,0.f,0.f, 0.3f), 0.01f * (0.010 + demo.musicVolume * 0.03f), false);
						glPopMatrix();
		    		}
		    		
					
		    		
		    	}
	    	}
	    	
	    	vec4f c1 = vec4f(1.0f,1.0f,0.95f, 1.f);
	    	vec4f c2 = vec4f(1.0f,1.0f,0.95f, 1.f);
	    	
	    	GL.modelviewMatrix = mat4f.IDENTITY;
	    	GL.projectionMatrix =  mat4f.scale(vec3f(1.f / demo.ratio, 1.f, 1.f));
			
		    auto width = 1.0f + 0.1f * demo.musicVolume;
	    	float fbeat = localTime / timePerBeat;
			int beat = localBeat;
			if ((4 <= beat) && (beat < 8)) 
			{
				auto t = 2.f * (1.f - smoothStep(4.f,5.f,fbeat)) - 2.f * (smoothStep(7.f,8.f,fbeat));
				t = lerp(-2.f, 2.f, smoothStep(-2.f,2.f, t));
				demo.bitmapFont.drawString(t, 0.33f, width, c1, c2, "#ponce");
				demo.bitmapFont.drawString(-t, -0.33f, width, c1, c2, "code");
			}
			
			if ((8 <= beat) && (beat < 12)) 
			{
				auto t = 2.f * (1.f - smoothStep(8.f,9.f,fbeat)) - 2.f * (smoothStep(11.f,12.f,fbeat));
				t = lerp(-2.f, 2.f, smoothStep(-2.f,2.f, t));
				demo.bitmapFont.drawString(t, 0.33f, width, c1, c2, "Graindolium");
				demo.bitmapFont.drawString(-t, -0.33f, width, c1, c2, "gfx/tools");
			}
			
			if ((12 <= beat) && (beat < 16)) 
			{
				auto t = 2.f * (1.f - smoothStep(12.f,13.f,fbeat)) - 2.f * (smoothStep(15.f,16.f,fbeat));
				t = lerp(-2.f, 2.f, smoothStep(-2.f,2.f, t));
				demo.bitmapFont.drawString(t, 0.33f, width, c1, c2, "wullon");
				demo.bitmapFont.drawString(-t, -0.33f, width, c1, c2, "music/design");
			}
			
			if ((16 <= beat) && (beat < 20)) 
			{
				auto t = 2.f * (1.f - smoothStep(16.f,17.f,fbeat)) - 2.f * (smoothStep(19.f,20.f,fbeat));
				t = lerp(-2.f, 2.f, smoothStep(-2.f,2.f, t));
				demo.bitmapFont.drawString(t, 0.33f, width, c1, c2, "flngr + wjk");
				demo.bitmapFont.drawString(-t, -0.33f, width, c1, c2, "code/help");
			}			
				
			if ((20 <= beat) && (beat < 24)) 
			{
				auto t = 2.f * (1.f - smoothStep(20.f,21.f,fbeat)) - 2.f * (smoothStep(23.f,24.f,fbeat));
				t = lerp(-2.f, 2.f, smoothStep(-2.f,2.f, t));
				demo.bitmapFont.drawString(t, 0.33f, width * 0.8, c1, c2, "lohrh");
				demo.bitmapFont.drawString(-t, -0.33f, width * 0.6, c1, c2, "gfx");
			}
			
			if ((24 <= beat) && (beat < 28)) 
			{
				auto t = 2.f * (1.f - smoothStep(24.f,25.f,fbeat)) - 2.f * (smoothStep(27.f,28.f,fbeat));
				t = lerp(-2.f, 2.f, smoothStep(-2.f,2.f, t));
				demo.bitmapFont.drawString(0.f, -t, width, c1, c2, "we like");
			}
			
			
		}
		
		void setGDO(GDOLogo logo)
		{
			if (logo is null)
			{
				setNawak();
				
			} else
			if (m_current !is logo)
			{
				m_current = logo;
				m_particleSet.target(logo);
				isNawak = false;
			}
		}	
		
		bool isNawak = false;
		
		void setNawak()
		{
	/*		if (isNawak)
			{
				return;
			}
			
			m_particleSet.nawak();
			m_current = null;			
			isNawak = true;
			assert(false);*/
			return ;
		}
		
		bool disorganized()
		{
			auto beat = localBeat();
			if (beat <= 32) return false;
			int r = (beat - 32) % 4;
			return (r == 3);
		}	
		
		bool validation()
		{
			auto beat = localBeat();
			if (beat <= 32) return false;
			int r = (beat - 32) % 4;
			return (r == 2);
		}		
		
		GDOLogo getCurrentTarget()
		{
			auto beat = localBeat();
			if (beat >= 32)
			{
				int r = (beat - 32) % 4;
				int l = (beat - 32) / 4;
				
				if (l < m_logos.length) return m_logos[l];
				else return null;
			} else return null;
		}	
		
		override void onMove(double dt) // move effect
		{			
			float speed = (1.f + 1.f * smoothStep(15.5 * timePerBeat, 16.5 * timePerBeat, localTime)
			                   - 1.f * smoothStep(16.5 * timePerBeat, 29.5 * timePerBeat, localTime)
			                   + 1.f * smoothStep(29.5 * timePerBeat, 30.5 * timePerBeat, localTime));
			m_tunnelProgression += dt * speed;	
			
			float noise = 0.f;
			
			float fbeat = localTime / timePerBeat();
			if (disorganized()) noise  = 10.f * frac(fbeat);
			
			GDOLogo target = getCurrentTarget();			
			setGDO(target);
			
			m_particleSet.move(dt, noise);
		}
	}
}

