/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module gl.fbo;


private import derelict.opengl.gl;	
private import derelict.opengl.glu;	
private import derelict.opengl.gl20;
private import derelict.opengl.extension.ext.framebuffer_object;
private import gl.renderbuffer;
private import gl.texture;
private import gl.state;
private import util.all;

/**
 * Un FBO, c'est un objet auquel on dit dans quelle buffer/texture on va rendre
 * 
 */


class FBO 
{
	public
	{
		enum Component { NONE = 0,
		                 RED = 1, 
		                 GREEN = 2,
		                 BLUE = 4, 
		                 ALPHA = 8,
		                 COLORS = RED | GREEN | BLUE | ALPHA,
		                 DEPTH = 16,
		                 ALL =  31}; // no stencil support now	
	}
	
	private 
	{
		
		class Attachment 
		{
			
			private 
			{
				
				FBO fb;
				GLint type, target, level, offset;
				union 
				{
					Texture tex;
					RenderBuffer rb;
				}
					
			}
			
			public 
			{
				
				this(FBO fb, GLint type) 
				{
					
					this.fb = fb;
					this.type = type;
					tex = null;
					rb = null;		
					target = -1;
					
				}		
				
				final bool assigned()
				{
					return (target != -1);
				}	
		
				final void attach()
				{
					
					if (assigned())
					{
						fb.bind();
						switch(target)
						{						
							case GL_TEXTURE_1D:
								glFramebufferTexture1DEXT(GL_FRAMEBUFFER_EXT, type, GL_TEXTURE_1D, tex.handle, level);
								break;
								
							case GL_TEXTURE_2D:
						    	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, type, GL_TEXTURE_2D, tex.handle, level);
						    	break;
						    	
						    case GL_TEXTURE_3D:
						    	glFramebufferTexture3DEXT(GL_FRAMEBUFFER_EXT, type, GL_TEXTURE_3D, tex.handle, level, offset);
						    	break;
						    	
						   case GL_RENDERBUFFER_EXT:
						    	glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, type, GL_RENDERBUFFER_EXT, rb.getHandle());
						    	break;
						    	
						   default:						
						}
						assert(GL.check());					
					}
					
				}
				
				final void setTarget(RenderBuffer rb) {
					
					this.rb = rb;
					target = GL_RENDERBUFFER_EXT;
					attach();									
					
				}
				
				
				final void setTarget(Texture1D t, int level) {
					
					this.level = level;
					this.tex = t;
					target = GL_TEXTURE_1D;									
					attach();
					
				}
				
				
				final void setTarget(Texture2D t, int level) {
					
					this.level = level;
					this.tex = t;
					target = GL_TEXTURE_2D;	
					attach();								
					
				}
				
				
				final void setTarget(Texture3D t, int level, int offset) {
					
					this.level = level;
					this.offset = offset;
					this.tex = t;
					target = GL_TEXTURE_3D;		
					attach();		
					
				}
        	
			}
				
			
		}
		
		GLuint handle;
			
		static FBO binded = null;
		static FBO current = null;
		static FBO defaultInstance = null;
		
		Component toWrite;
		bool m_main;
				
		final void bind() 
		{
			if (binded !is this) 
			{
				glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, handle);	
				assert(GL.check());
				binded = this;
				
			}
		}
		
		
		this(bool main) 
		{
			m_main = main;
			if (main) 
			{				
				handle = 0;
				color.length = 1;
				color[0] = new Attachment(this,  GL_BACK);
				stencil = null;
				depth = null;
				setWrite(Component.RED | Component.GREEN | Component.BLUE | Component.ALPHA);
				
			} else 
			{				
				glGenFramebuffersEXT(1, &handle);
				assert(GL.check());
				color.length = maxColorAttachments();
				
				for (int i = 0; i < color.length; i++) 
				{				
					color[i] = new Attachment(this,  GL_COLOR_ATTACHMENT0_EXT + i);					
				}							
				
				stencil = new Attachment(this, GL_STENCIL_ATTACHMENT_EXT);			
				depth = new Attachment(this, GL_DEPTH_ATTACHMENT_EXT);
				setWrite(Component.RED | Component.GREEN | Component.BLUE | Component.ALPHA);
			}
			
		}
		
	}
	
	public {
		
		Attachment[] color;
		Attachment stencil;
		Attachment depth;
		
		
		this() 
		{
			this(false);				
		}		
		
		~this() {
			
			if (binded is this) binded = null;
			if (current is this) current = null;
    		
    		if (handle != 0) 
    		{
    			glDeleteFramebuffersEXT(1, &handle);
			}
			assert(check());
        	assert(GL.check());
			
		}
		
		final void setWrite(Component c)
		{
			toWrite = c;	
		}
		
		static final FBO getDefault()
		{
			if (defaultInstance is null)
			{
				defaultInstance = new FBO(true);
			}
			return defaultInstance;	
		}
		
		
		static final int maxColorAttachments() {
			
			int max;
			glGetIntegerv(GL_MAX_COLOR_ATTACHMENTS_EXT,&max);
			return max;
			
		}
		
		final void use() 
		{
			bind();
					
			if (this !is current) 
			{
				
				static assert(GL_TRUE == 1);
				static assert(GL_FALSE == 0);
				
				glColorMask(cast(bool)(toWrite & Component.RED), 
				            cast(bool)(toWrite & Component.GREEN), 
				            cast(bool)(toWrite & Component.BLUE), 
				            cast(bool)(toWrite & Component.ALPHA)); 
				glDepthMask(cast(bool)(toWrite & Component.DEPTH));		
				glStencilMask(0);		
				assert(GL.check());	
			/*	
				for (int i = 0; i < color.length; i++) 
				{				
					if (color[i] !is null) color[i].attach();
				}
				
				if (depth !is null) depth.attach();
				if (stencil !is null) stencil.attach();		
			*/	current = this;
				
				
			}		
			assert(GL.check());			
			assert(check());			
		}			
		
		final bool check()
		{
			GLenum status = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
			switch(status)
			{
				case GL_FRAMEBUFFER_COMPLETE_EXT:
					return true;
				
				case GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT:
					assert(false);
				
				case GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT:
				    assert(false);
					
    			case GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT:
    				assert(false);
					
    			case GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT:
    				assert(false);
					
    			case GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT:
    				assert(false);
					
   				case GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT:   					
					assert(false);
				
				case GL_FRAMEBUFFER_UNSUPPORTED_EXT:
					assert(false);
					
				default:
					assert(false);
				
			}
		}
		
		final GLenum getBuffer(int i)
		{
			if (m_main)
			{
				static auto const bufferLookup = [GL_BACK, GL_AUX0, GL_AUX1, GL_AUX2];
				return bufferLookup[i];
			} else
			{
				return ( GL_COLOR_ATTACHMENT0_EXT + i );				
			}
		}
		

		final void setDrawBuffers(int[] drawBuffers) // pass here which color buffers to write (in general : 0..3)
		{
			bind();
			assert(drawBuffers.length > 0);
			if (drawBuffers.length == 1)
			{
				glDrawBuffer(getBuffer(drawBuffers[0]));
			} else
			{			
    			GLenum ids[16];    		
    			int n = drawBuffers.length;    		
    			for (int i = 0; i < n; ++i) 
    			{
	    			ids[i] = getBuffer(drawBuffers[i]);
    			}
    			glDrawBuffers(n, ids.ptr);
			}

    		assert(GL.check());
		}
	}
}
