/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module sdl.app;

private import derelict.sdl.sdl;	
private import derelict.opengl.gl;	
private import derelict.opengl.glu;	
private import derelict.opengl.gl20;
private import derelict.opengl.gl20;
private import derelict.devil.il;
private import derelict.devil.ilu;
private import std.gc;
private import sdl.state;
private import std.stdio;
private import util.common;
private import util.framecounter;

   



enum FSAA { OFF, FSAA2X, FSAA4X, FSAA8X, FSAA16X };

class SDLApp
{

    protected
    {
	    bool m_fullscreen;
        bool m_finished;
        bool m_resizable;
        int m_width, m_height;
        
        SDL m_sdl = null;

        SDL_Surface* m_screen;
        FrameCounter m_frameCounter = null;
        
        int m_mousex, m_mousey, m_ancmousex, m_ancmousey;
        
    }
    
    private
    {	
	    static const GLint[FSAA.max + 1] FSAA_toInt = [-1,2,4,8,16];
    }

    public
    {
		auto const AUTO_DETECT = -1; // a utiliser a la place de width et height pour utiliser la taille courante de l'ecran

        this(int width,  // desired window width, could be AUTO_DETECT to match current resolution
             int height, // desired window height, could be AUTO_DETECT to match current resolution
             bool fullscreen, // fullscreen or not
             bool resizable, // resizeable or not
             char[] initTitle, 
             char[] iconFile, // can be null
             FSAA fsaa, 
             int wantedFrameRate) // if wantedFrameRate == 0 then the frame counter will count frames to find the frame rate
        {
	        m_sdl = SDL.instance; // init SDL if required
            m_fullscreen = fullscreen; 
            m_resizable = resizable & (! fullscreen);
            
            
            SDL_VideoInfo* videoInfo = SDL_GetVideoInfo();
            
            m_width = (width == AUTO_DETECT) ? videoInfo.current_w : width;
            m_height = (height == AUTO_DETECT) ? videoInfo.current_h : height;
          
            uint flags = SDL_DOUBLEBUF | SDL_HWSURFACE | SDL_OPENGL;
            if (resizable) flags |= SDL_RESIZABLE;
            if (fullscreen) flags |= SDL_FULLSCREEN;
            
            if (iconFile !is null)
            {
            	SDL_Surface* icon = SDL_LoadBMP(toCString(iconFile));
				SDL_WM_SetIcon(icon, null);
			}
			
			
			SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8);
			SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 8);
			SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 8);
			SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, 8);
		//	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 24);
			SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

			if (fsaa != FSAA.OFF)
			{
				SDL_GL_SetAttribute (SDL_GL_MULTISAMPLESAMPLES, FSAA_toInt[fsaa] );
				SDL_GL_SetAttribute (SDL_GL_MULTISAMPLEBUFFERS,1);
			}
			
			

            // Create the screen surface (ie window)
            m_screen = SDL_SetVideoMode(m_width,
                                        m_height,
                                        SDL_GetVideoInfo().vfmt.BitsPerPixel,
                                        flags);
                                        
			title = initTitle;                                      

            DerelictGL.load();
            DerelictGL.loadVersions(GLVersion.Version20);
            DerelictGL.loadExtensions();
            
            DerelictGLU.load();
            
            if (fsaa != FSAA.OFF) glEnable(GL_MULTISAMPLE);
            //else glDisable(GL_MULTISAMPLE);
            
            DerelictIL.load();
            DerelictILU.load();
		    ilInit();
		    iluInit();            
		    
		    if (wantedFrameRate > 0)
		    {
			    m_frameCounter = new FixedFrameCounter(wantedFrameRate);
			}
		    else 
		    {
			    m_frameCounter = new VariableFrameCounter(1.0);
			}
            m_finished = false;            			
        }
        
        final int width()
        {
	     	return m_width;   
        }
        
        final int height()
        {
	     	return m_height;   
        }        

        final char[] title(char[] s)
        {
            SDL_WM_SetCaption(toCString(s), null);
            return s;
        }

        final void terminate()
        {
            m_finished = true;
        }
        
        final void swapBuffers()
        {
	        SDL_GL_SwapBuffers();
        }
        
        final void frame(bool checkEvents)
        {
	        if (checkEvents) processEvents();
	        
	        if (!m_finished)
	        {		
		        if (m_frameCounter.tick())
	            {
	            	OnFrameRateChanged(m_frameCounter.frameRate);                    
	        	}
	        		
		        final double time = m_frameCounter.elapsedTime;
		       	if (!m_finished) OnMove(time, m_frameCounter.deltaTime);		       	
	        	if (!m_finished) OnRender(time);
	
	           
        	}        	
        }

        final void run()
        {
 			while (! m_finished)
            {
	            frame(true);
            }            
        }


        final void processEvents()
        {
			SDL_Event event;
            while (SDL_PollEvent(&event))
            {
                switch (event.type)
                {
                    // handle keyboard

	                case SDL_KEYUP:
	                {
	                    SDLKey key = event.key.keysym.sym;
	                    SDL.instance.getKeyboard().markAsPressed( key );
	                    OnKeyUp( key );
	                    break;
                    }
	
	                case SDL_KEYDOWN:
	                {
		                SDLKey key = event.key.keysym.sym;
	                    SDL.instance.getKeyboard().markAsReleased( key );
	                    OnKeyDown( key );
	                    break;	                    
                    }
                    
                    // handle mouse
	
	                case SDL_MOUSEMOTION:
	                	m_ancmousex = m_mousex;
	                    m_ancmousey = m_mousey;
	                    m_mousex = event.motion.x;
	                    m_mousey = event.motion.y;	                   
	                    
	                    OnMouseMove(m_mousex, m_mousey, m_mousex - m_ancmousex, m_mousey - m_ancmousey);
	                    break;
	                    
					case SDL_MOUSEBUTTONDOWN:
						OnMouseDown(event.button.button);
	                    break;
	                    
					case SDL_MOUSEBUTTONUP:
						OnMouseUp(event.button.button);
						break;
						
					case SDL_JOYAXISMOTION:
						int joy_index = event.jaxis.which;
						int axis_index = event.jaxis.axis;
						float value = event.jaxis.value / cast(float)(short.max);
						SDL.instance.getJoystick(joy_index).setAxis(axis_index, value);
						break;
					
					case SDL_JOYBUTTONUP:	
					case SDL_JOYBUTTONDOWN:
					{
						int joy_index = event.jbutton.which;
						int button_index = event.jbutton.button;
						bool value = (event.jbutton.state == SDL_PRESSED);
						SDL.instance.getJoystick(joy_index).setButton(button_index, value);
						break;
					}
	
	                case SDL_QUIT:
	                    terminate();
	                    break;
	                    
					case SDL_VIDEORESIZE:
						m_width = event.resize.w;
						m_height = event.resize.h;
						OnReshape(event.resize.w,event.resize.h);	
						break;
						
					default:
	                    break;
            	}
            }
        }

        // events handlers
        
        abstract void OnRender(double elapsedTime);
        abstract void OnMove(double elapsedTime, float dt);
        abstract void OnKeyUp(int key);
        abstract void OnKeyDown(int key);                
        abstract void OnMouseMove(int x, int y, int dx, int dy);
        abstract void OnMouseDown(int button);
        abstract void OnMouseUp(int button);        
        abstract void OnReshape(int width, int height);
        abstract void OnFrameRateChanged(float frameRate);

    }
}


