/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module sdl.state;

private import derelict.sdl.sdl;
private import util.singleton;
private import sdl.keyboard;
private import std.stdio;
private import sdl.joystick;

class SDL
{
	public class Error : object.Exception
	{	
		this(char[] message)
		{
			super(message);	
		}
	}
	
	mixin Singleton;
	
	private
	{
		SDLKeyboard kb;
		SDLJoystick[] joy;
		
		bool subSystemInitialized(int subSystem)
		{
			int inited = SDL_WasInit(SDL_INIT_EVERYTHING);
			return 0 != ( inited & subSystem );
		}
		
		
		
		
		this()
		{
			DerelictSDL.load();
			
			if (0 != SDL_Init(0)) throw new SDL.Error("Unable to initialize SDL.");			
			
			subSystemInit(SDL_INIT_TIMER);
			subSystemInit(SDL_INIT_VIDEO);
			subSystemInit(SDL_INIT_JOYSTICK);
			
			
			
			// initialize joysticks, if any
			joy.length = SDL_NumJoysticks();
			for (int i = 0; i < joy.length; i++)
			{
				joy[i] = new SDLJoystick(i);	
			}			
			
			kb = new SDLKeyboard();
			
			SDL_EventState ( cast(ubyte) SDL_ALLEVENTS, SDL_ENABLE);			
		}
		
		~this()
		{
			debug(2)writefln(">deleting SDL");
			for (int i = 0; i < joy.length; i++)
			{
				delete joy[i];
			}
			delete joy;
			delete kb;
			SDL_Quit();			
			debug(2) writefln("<deleting SDL");
		}
	}	
	
	public
	{	
		SDLKeyboard getKeyboard()
		{
			return kb;
		}
		
		SDLJoystick getJoystick(int i)
		{
			return joy[i];
		}
		
		int getNumJoysticks()
		{
			return joy.length;	
		}
		
		void subSystemInit(int flag)
		{
			if (! subSystemInitialized(flag))
			{
				int res = SDL_InitSubSystem(flag);
				if (0 != res) throw new SDL.Error("Unable to initialize a SDL subsystem.");			
			}			
		}
		
		void subSystemQuit(int flag)
		{
			SDL_QuitSubSystem(flag);
		}
		
	}
	
}