/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module util.geometry.curve;

// A generic curve providing different interpolation
// indexed with float

import util.math;
import util.vector;

class Curve(T)
{
	public
	{
		enum Sampling
		{
			NEAREST,
			LINEAR,
			HERMITE
		}	
	}
	
	private
	{
		Vector!(T) m_points; // not especially points, could be vecX, matX, quat...	
		
		int clampIndex(int i)
		{
			return max(0, min(i, m_points.size - 1));
		}
		
		T hermite(float frac_pos, T xm1, T x0, T x1, T x2)
		{
		   auto final c = (x1 - xm1) * 0.5f;
		   auto final v = x0 - x1;
		   auto final w = c + v;
		   auto final a = w + v + (x2 - x0) * 0.5f;
		   auto final b_neg = w + a;		
		   return ((((a * frac_pos) - b_neg) * frac_pos + c) * frac_pos + x0);
		}
	}
	
	public
	{
		this()
		{
			m_points = new Vector!(T);
		}
		
		this(T[] a)
		{
			this();
			m_points.addArray(a);
		}
		
		this(Vector!(T) v)
		{
			this();
			m_points.addAll(v);
		}
		
		Vector!(T) points()
		{
			return m_points;
		}
		
		// x is a float index clamped to [0..count-1]
		T sample(float x, Sampling sampling)
		{
			switch(sampling)
			{	
				case Sampling.LINEAR:
				{
					final int i_part = cast(int) floor(x);	
					final float f_part = x - i_part;
					final auto d1 = m_points[clampIndex(i_part)];
					final auto d2 = m_points[clampIndex(i_part + 1)];
					return d1 * (1.f - f_part) + d2 * f_part;
				}
				
				case Sampling.HERMITE:
				{
					final int i_part = cast(int) floor(x);	
					final float f_part = x - i_part;
					final T xm1 = m_points[clampIndex(i_part - 1)];
					final T x0 = m_points[clampIndex(i_part + 0)];
					final T x1 = m_points[clampIndex(i_part + 1)];
					final T x2 = m_points[clampIndex(i_part + 2)];
					return hermite(f_part, xm1, x0, x1, x2);
				}
				
				case Sampling.NEAREST:
				default: 
				{
					return m_points[clampIndex(round(x))];	
				}
			}	
		}		
		
		int length()
		{
			return m_points.size();	
		}
		
		T * ptr()
		{
			return m_points.ptr();
		}
		
	}
}