/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module util.geometry.mat3;

private import util.geometry.all;
private import util.math;
private import util.common;



final struct mat3(T) 
{	
	public {
		
		union 
		{
			T c[3][3];
			T[9] v;
			T c00, c01, c02, c10, c11, c12, c20, c21, c22;			
		}	
		
		static mat3 opCall(T c00, T c01, T c02, 
		                   T c10, T c11, T c12, 
		                   T c20, T c21, T c22 ) 
		{			
			mat3 m;
			m.c[0][0] = c00;	m.c[0][1] = c01;	m.c[0][2] = c02;
			m.c[1][0] = c10;	m.c[1][1] = c11;	m.c[1][2] = c12;
			m.c[2][0] = c20;	m.c[2][1] = c21;	m.c[2][2] = c22;	
			return m;
		}	
		
		mat3 opMul(mat3 m) 
		{			
			return mat3
			(
				c[0][0] * m.c[0][0] + c[0][1] * m.c[1][0] + c[0][2] * m.c[2][0],
				c[0][0] * m.c[0][1] + c[0][1] * m.c[1][1] + c[0][2] * m.c[2][1],
				c[0][0] * m.c[0][2] + c[0][1] * m.c[1][2] + c[0][2] * m.c[2][2],				
				
				c[1][0] * m.c[0][0] + c[1][1] * m.c[1][0] + c[1][2] * m.c[2][0],
				c[1][0] * m.c[0][1] + c[1][1] * m.c[1][1] + c[1][2] * m.c[2][1],
				c[1][0] * m.c[0][2] + c[1][1] * m.c[1][2] + c[1][2] * m.c[2][2],
				
				c[2][0] * m.c[0][0] + c[2][1] * m.c[1][0] + c[2][2] * m.c[2][0],
				c[2][0] * m.c[0][1] + c[2][1] * m.c[1][1] + c[2][2] * m.c[2][1],
				c[2][0] * m.c[0][2] + c[2][1] * m.c[1][2] + c[2][2] * m.c[2][2]
			);
		}
		
		vec3!(T) opMul(vec3!(T) v) {
			
			return vec3!(T)
			(
				c[0][0] * v.v[0] + c[0][1] * v.v[1] + c[0][2] * v.v[2],
				c[1][0] * v.v[0] + c[1][1] * v.v[1] + c[1][2] * v.v[2],
				c[2][0] * v.v[0] + c[2][1] * v.v[1] + c[2][2] * v.v[2]	
			);
			
		}
		
		mat3 opMul(T v) {
			
			return mat3(
				c[0][0] * v,
				c[0][1] * v,
				c[0][2] * v,
				c[1][0] * v,
				c[1][1] * v,
				c[1][2] * v,
				c[2][0] * v,
				c[2][1] * v,
				c[2][2] * v
			);
			
		}
		
		mat3 opAdd(mat3 v) 
		{			
			return mat3(
				c[0][0] + v.c[0][0],
				c[0][1] + v.c[0][1],
				c[0][2] + v.c[0][2],
				c[1][0] + v.c[1][0],
				c[1][1] + v.c[1][1],
				c[1][2] + v.c[1][2],
				c[2][0] + v.c[2][0],
				c[2][1] + v.c[2][1],
				c[2][2] + v.c[2][2]				
			);			
		}
		
		mat3 opSub(mat3 v) 
		{			
			return mat3(
				c[0][0] - v.c[0][0],
				c[0][1] - v.c[0][1],
				c[0][2] - v.c[0][2],
				c[1][0] - v.c[1][0],
				c[1][1] - v.c[1][1],
				c[1][2] - v.c[1][2],
				c[2][0] - v.c[2][0],
				c[2][1] - v.c[2][1],
				c[2][2] - v.c[2][2]			
			);
		}
		

		void opAddAssign(mat3 v) 
		{			
			c[0][0] += v.c[0][0];
			c[0][1] += v.c[0][1];
			c[0][2] += v.c[0][2];
			c[1][0] += v.c[1][0];
			c[1][1] += v.c[1][1];
			c[1][2] += v.c[1][2];
			c[2][0] += v.c[2][0];
			c[2][1] += v.c[2][1];
			c[2][2] += v.c[2][2];
		}
		
		
		void opSubAssign(mat3 v) 
		{			
			c[0][0] -= v.c[0][0];
			c[0][1] -= v.c[0][1];
			c[0][2] -= v.c[0][2];
			c[1][0] -= v.c[1][0];
			c[1][1] -= v.c[1][1];
			c[1][2] -= v.c[1][2];
			c[2][0] -= v.c[2][0];
			c[2][1] -= v.c[2][1];
			c[2][2] -= v.c[2][2];
		}
		
		mat3 transposed() 
		{			
			return mat3
			(
				c[0][0], c[1][0], c[2][0],
				c[0][1], c[1][1], c[2][1],
				c[0][2], c[1][2], c[2][2]				
			);
			
		}
		
		void transpose() 
		{			
			swap(c[0][1],c[1][0]);
			swap(c[0][2],c[2][0]);			
			swap(c[1][2],c[2][1]);
		}
		
		char[] toString() 
		{			
			alias std.string.toString s;
			
			return "((" ~ s(c[0][0]) ~ ", " ~ s(c[0][1]) ~ ", " ~ s(c[0][2]) ~ ")"\n
			      ~" (" ~ s(c[1][0]) ~ ", " ~ s(c[1][1]) ~ ", " ~ s(c[1][2]) ~ ")"\n
			      ~" (" ~ s(c[2][0]) ~ ", " ~ s(c[2][1]) ~ ", " ~ s(c[2][2]) ~ "))";
			
		}
		
		static {
			
			mat3 zero() 
			{				
				return mat3
				(
					0, 0, 0,
                	0, 0, 0,
                	0, 0, 0
                );
			}
			
			mat3 identity() 
			{				
				return mat3
				(
					1, 0, 0,
                	0, 1, 0,
                	0, 0, 1
                );	
			}
			
			mat3 scale(T f) 
			{			
				return mat3
				(
					f, 0, 0, 
                	0, f, 0,
                	0, 0, 1
                );	
				
			}
			
			mat3 scale(T x, T y) 
			{			
				return mat3
				(
					x, 0, 0,
                	0, y, 0,
                	0, 0, 1
                );	
				
			}
			
			mat3 scale(vec2!(T) v) 
			{			
				return mat3
				(
					v.x,   0,   0,
                	  0, v.y,   0,
                	  0,   0,   1                	  
                );					
			}			
			
			mat3 translate(T x, T y) 
			{			
				return mat3
				(
					1, 0, x,
                	0, 1, y,
                	0, 0, 1
                );	
				
			}			
			
			mat3 translate(vec2!(T) v) 
			{			
				return mat3
				(
					1, 0, v.x,
                	0, 1, v.y,
                	0, 0, 1                	
                );					
			}			
			
			mat3 rotateX(T angle) 
			{				
				T cosa = cos(angle);
				T sina = sin(angle);			
				return mat3
				(
					1,    0,     0,
                	0, cosa, -sina,
                	0, sina,  cosa
                );				
			}
			
			
			mat3 rotateY(T angle) 
			{				
				T cosa = cos(angle);
				T sina = sin(angle);			
				return mat3
				(
					cosa, 0, -sina,
                	   0, 1,     0,
                	sina, 0,  cosa               	   
                );				
			}
			
			
			mat3 rotateZ(T angle) 
			{				
				T cosa = cos(angle);
				T sina = sin(angle);			
				return mat3
				(
					cosa, -sina, 0,
                	sina,  cosa, 0,
                	   0,     0, 1                	   
                );				
			}
		}	
	}
}

alias mat3!(float) mat3f;
alias mat3!(double) mat3d;
