/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module util.geometry.mat4;

private import util.geometry.all;
private import util.math;
private import util.common;

final struct mat4(T) 
{	
	public 
	{
		
		union 
		{
			T[4][4] c;			
		}	
		
		static mat4 opCall(T c00, T c01, T c02, T c03,
		                   T c10, T c11, T c12, T c13,
		                   T c20, T c21, T c22, T c23,
		                   T c30, T c31, T c32, T c33) 
		{		
			mat4 m;	
			m.c[0][0] = c00;	m.c[0][1] = c01;	m.c[0][2] = c02;	m.c[0][3] = c03;
			m.c[1][0] = c10;	m.c[1][1] = c11;	m.c[1][2] = c12;	m.c[1][3] = c13;
			m.c[2][0] = c20;	m.c[2][1] = c21;	m.c[2][2] = c22;	m.c[2][3] = c23;
			m.c[3][0] = c30;	m.c[3][1] = c31;	m.c[3][2] = c32;	m.c[3][3] = c33;
			return m;
		}
		
		static mat4 opCall(T x)
		{			
			return mat4(x, x, x, x, x, x, x, x, x, x, x, x, x, x, x, x);
		}
		
		static mat4 IDENTITY, ZERO;
		
		static this()
		{
			ZERO = mat4(0);
			IDENTITY = mat4(1, 0, 0, 0,
			                0, 1, 0, 0,
			                0, 0, 1, 0,
			                0, 0, 0, 1);				
		}	
		
		final T * ptr()
		{
			return &c[0][0];	
		}
		
		mat4 opMul(mat4 m) {
			
			return mat4(
				c[0][0] * m.c[0][0] + c[0][1] * m.c[1][0] + c[0][2] * m.c[2][0] + c[0][3] * m.c[3][0],
				c[0][0] * m.c[0][1] + c[0][1] * m.c[1][1] + c[0][2] * m.c[2][1] + c[0][3] * m.c[3][1],
				c[0][0] * m.c[0][2] + c[0][1] * m.c[1][2] + c[0][2] * m.c[2][2] + c[0][3] * m.c[3][2],
				c[0][0] * m.c[0][3] + c[0][1] * m.c[1][3] + c[0][2] * m.c[2][3] + c[0][3] * m.c[3][3],
				
				c[1][0] * m.c[0][0] + c[1][1] * m.c[1][0] + c[1][2] * m.c[2][0] + c[1][3] * m.c[3][0],
				c[1][0] * m.c[0][1] + c[1][1] * m.c[1][1] + c[1][2] * m.c[2][1] + c[1][3] * m.c[3][1],
				c[1][0] * m.c[0][2] + c[1][1] * m.c[1][2] + c[1][2] * m.c[2][2] + c[1][3] * m.c[3][2],
				c[1][0] * m.c[0][3] + c[1][1] * m.c[1][3] + c[1][2] * m.c[2][3] + c[1][3] * m.c[3][3],
				
				c[2][0] * m.c[0][0] + c[2][1] * m.c[1][0] + c[2][2] * m.c[2][0] + c[2][3] * m.c[3][0],
				c[2][0] * m.c[0][1] + c[2][1] * m.c[1][1] + c[2][2] * m.c[2][1] + c[2][3] * m.c[3][1],
				c[2][0] * m.c[0][2] + c[2][1] * m.c[1][2] + c[2][2] * m.c[2][2] + c[2][3] * m.c[3][2],
				c[2][0] * m.c[0][3] + c[2][1] * m.c[1][3] + c[2][2] * m.c[2][3] + c[2][3] * m.c[3][3],
				
				c[3][0] * m.c[0][0] + c[3][1] * m.c[1][0] + c[3][2] * m.c[2][0] + c[3][3] * m.c[3][0],
				c[3][0] * m.c[0][1] + c[3][1] * m.c[1][1] + c[3][2] * m.c[2][1] + c[3][3] * m.c[3][1],
				c[3][0] * m.c[0][2] + c[3][1] * m.c[1][2] + c[3][2] * m.c[2][2] + c[3][3] * m.c[3][2],
				c[3][0] * m.c[0][3] + c[3][1] * m.c[1][3] + c[3][2] * m.c[2][3] + c[3][3] * m.c[3][3]
			);
			
		}
		
		vec4!(T) opMul(vec4!(T) v) {
			
			return vec4!(T)(
				c[0][0] * v.v[0] + c[0][1] * v.v[1] + c[0][2] * v.v[2] + c[0][3] * v.v[3],
				c[1][0] * v.v[0] + c[1][1] * v.v[1] + c[1][2] * v.v[2] + c[1][3] * v.v[3],
				c[2][0] * v.v[0] + c[2][1] * v.v[1] + c[2][2] * v.v[2] + c[2][3] * v.v[3],
				c[3][0] * v.v[0] + c[3][1] * v.v[1] + c[3][2] * v.v[2] + c[3][3] * v.v[3]				
			);
			
		}
		
		mat4 opMul(T v) {
			
			return mat4(
				c[0][0] * v,
				c[0][1] * v,
				c[0][2] * v,
				c[0][3] * v,
				c[1][0] * v,
				c[1][1] * v,
				c[1][2] * v,
				c[1][3] * v,
				c[2][0] * v,
				c[2][1] * v,
				c[2][2] * v,
				c[2][3] * v,
				c[3][0] * v,
				c[3][1] * v,
				c[3][2] * v,
				c[3][3] * v
			);
			
		}
		
		mat4 opAdd(mat4 v) {
			
			return mat4(
				c[0][0] + v.c[0][0],
				c[0][1] + v.c[0][1],
				c[0][2] + v.c[0][2],
				c[0][3] + v.c[0][3],
				c[1][0] + v.c[1][0],
				c[1][1] + v.c[1][1],
				c[1][2] + v.c[1][2],
				c[1][3] + v.c[1][3],
				c[2][0] + v.c[2][0],
				c[2][1] + v.c[2][1],
				c[2][2] + v.c[2][2],
				c[2][3] + v.c[2][3],
				c[3][0] + v.c[3][0],
				c[3][1] + v.c[3][1],
				c[3][2] + v.c[3][2],
				c[3][3] + v.c[3][3]
			);
			
		}
		
		mat4 opSub(mat4 v) {
			
			return mat4(
				c[0][0] - v.c[0][0],
				c[0][1] - v.c[0][1],
				c[0][2] - v.c[0][2],
				c[0][3] - v.c[0][3],
				c[1][0] - v.c[1][0],
				c[1][1] - v.c[1][1],
				c[1][2] - v.c[1][2],
				c[1][3] - v.c[1][3],
				c[2][0] - v.c[2][0],
				c[2][1] - v.c[2][1],
				c[2][2] - v.c[2][2],
				c[2][3] - v.c[2][3],
				c[3][0] - v.c[3][0],
				c[3][1] - v.c[3][1],
				c[3][2] - v.c[3][2],
				c[3][3] - v.c[3][3]
			);
			
		}
		

		void opAddAssign(mat4 v) {
			
			c[0][0] += v.c[0][0];
			c[0][1] += v.c[0][1];
			c[0][2] += v.c[0][2];
			c[0][3] += v.c[0][3];
			c[1][0] += v.c[1][0];
			c[1][1] += v.c[1][1];
			c[1][2] += v.c[1][2];
			c[1][3] += v.c[1][3];
			c[2][0] += v.c[2][0];
			c[2][1] += v.c[2][1];
			c[2][2] += v.c[2][2];
			c[2][3] += v.c[2][3];
			c[3][0] += v.c[3][0];
			c[3][1] += v.c[3][1];
			c[3][2] += v.c[3][2];
			c[3][3] += v.c[3][3];			
			
		}
		
		
		void opSubAssign(mat4 v) {
			
			c[0][0] -= v.c[0][0];
			c[0][1] -= v.c[0][1];
			c[0][2] -= v.c[0][2];
			c[0][3] -= v.c[0][3];
			c[1][0] -= v.c[1][0];
			c[1][1] -= v.c[1][1];
			c[1][2] -= v.c[1][2];
			c[1][3] -= v.c[1][3];
			c[2][0] -= v.c[2][0];
			c[2][1] -= v.c[2][1];
			c[2][2] -= v.c[2][2];
			c[2][3] -= v.c[2][3];
			c[3][0] -= v.c[3][0];
			c[3][1] -= v.c[3][1];
			c[3][2] -= v.c[3][2];
			c[3][3] -= v.c[3][3];
			
		}
		
		mat4 transposed() {
			
			return mat4(
				c[0][0], c[1][0], c[2][0], c[3][0],
				c[0][1], c[1][1], c[2][1], c[3][1],
				c[0][2], c[1][2], c[2][2], c[3][2],
				c[0][3], c[1][3], c[2][3], c[3][3]
			);
			
		}
		
		/**
		 * in-place transpose
		 */
		void transpose() 
		{			
			swap(c[0][1],c[1][0]);
			swap(c[0][2],c[2][0]);
			swap(c[0][3],c[3][0]);
			swap(c[1][2],c[2][1]);
			swap(c[1][3],c[3][1]);
			swap(c[2][3],c[3][2]);			
		}
		

		T minor (int r0, int r1, int r2, int c0, int c1, int c2) {
		
			return c[r0][c0] * (c[r1][c1] * c[r2][c2] - c[r2][c1] * c[r1][c2]) -
			       c[r0][c1] * (c[r1][c0] * c[r2][c2] - c[r2][c0] * c[r1][c2]) +
				   c[r0][c2] * (c[r1][c0] * c[r2][c1] - c[r2][c0] * c[r1][c1]);
		}
		
		T determinant() {
		
			return c[0][0] * minor( 1, 2, 3, 1, 2, 3) -
			       c[0][1] * minor( 1, 2, 3, 0, 2, 3) +
			       c[0][2] * minor( 1, 2, 3, 0, 1, 3) -
			       c[0][3] * minor( 1, 2, 3, 0, 1, 2);
		   
		}
		
		
		mat4 adjoint() {
			
			return mat4(
				minor( 1, 2, 3, 1, 2, 3),
				-minor( 0, 2, 3, 1, 2, 3),
				minor( 0, 1, 3, 1, 2, 3),
				-minor( 0, 1, 2, 1, 2, 3),
				
				-minor( 1, 2, 3, 0, 2, 3),
				minor( 0, 2, 3, 0, 2, 3),
				-minor( 0, 1, 3, 0, 2, 3),
				minor( 0, 1, 2, 0, 2, 3),
				
				minor( 1, 2, 3, 0, 1, 3),
				-minor( 0, 2, 3, 0, 1, 3),
				minor( 0, 1, 3, 0, 1, 3),
				-minor( 0, 1, 2, 0, 1, 3),
				
				-minor( 1, 2, 3, 0, 1, 2),
				minor( 0, 2, 3, 0, 1, 2),
				-minor( 0, 1, 3, 0, 1, 2),
				minor( 0, 1, 2, 0, 1, 2)
			);
			
		}
		
		mat4 inverted() 
		{			
			return adjoint() * (1.0f / determinant());		    
		}
		
		
		
		char[] toString() 
		{						
			/*return format( "( ( %s, %s, %s, %s ),"\n"( %s, %s, %s, %s ),"\n"( %s, %s, %s, %s ),"\n"( %s, %s, %s, %s )  )", 
			               c00, c01, c02, c03, c10, c11, c12, c13, c20, c21, c22, c23, c30, c31, c32, c33);
			  */             
			alias std.string.toString s;
			
			return "((" ~ s(c[0][0]) ~ ", " ~ s(c[0][1]) ~ ", " ~ s(c[0][2]) ~ ", " ~ s(c[0][3]) ~ ")"\n
			      ~" (" ~ s(c[1][0]) ~ ", " ~ s(c[1][1]) ~ ", " ~ s(c[1][2]) ~ ", " ~ s(c[1][3]) ~ ")"\n
			      ~" (" ~ s(c[2][0]) ~ ", " ~ s(c[2][1]) ~ ", " ~ s(c[2][2]) ~ ", " ~ s(c[2][3]) ~ ")"\n
			      ~" (" ~ s(c[3][0]) ~ ", " ~ s(c[3][1]) ~ ", " ~ s(c[3][2]) ~ ", " ~ s(c[3][3]) ~ "))";			               
		}
		
		// "basic" matrixes used in 3D
		static 
		{
			
			mat4 scale(vec3!(T) v) {
			
				return mat4(
					v.x,   0,   0, 0,
                	  0, v.y,   0, 0,
                	  0,   0, v.z, 0,
                	  0,   0,   0, 1
                );	
				
			}
			
			mat4 scale(vec4!(T) v) {
			
				return mat4(
					v.x,   0,   0, 0,
                	  0, v.y,   0, 0,
                	  0,   0, v.z, 0,
                	  0,   0,   0, v.w
                );
			}
			
			mat4 translate(vec3!(T) v) 
			{			
				return mat4(
					1, 0, 0, v.x,
                	0, 1, 0, v.y,
                	0, 0, 1, v.z,
                	0, 0, 0, 1
                );					
			}
			
			
			mat4 rotateX(T angle) 
			{				
				T cosa = cos(angle);
				T sina = sin(angle);			
				return mat4(
					1,    0,     0, 0,
                	0, cosa, -sina, 0,
                	0, sina,  cosa, 0,
                	0,    0,     0, 1
                );					
			}
			
			
			mat4 rotateY(T angle) 
			{				
				T cosa = cos(angle);
				T sina = sin(angle);			
				return mat4(
					cosa, 0, -sina, 0,
                	   0, 1,     0, 0,
                	sina, 0,  cosa, 0,
                	   0, 0,     0, 1
                );					
			}
			
			
			mat4 rotateZ(T angle) {
				
				T cosa = cos(angle);
				T sina = sin(angle);			
				return mat4(
					cosa, -sina, 0, 0,
                	sina,  cosa, 0, 0,
                	   0,     0, 1, 0,
                	   0,     0, 0, 1
                );	
				
			}
			
			// fovy in degree
			mat4 perspective(T fovy, T aspect, T zNear, T zFar) {
				
				T f = 1 / tan( fovy * PI_D / 360);
				T d = 1 / (zNear - zFar);
				
				return mat4(
					f / aspect, 0,                  0,                    0,
					         0, f,                  0,                    0,
					         0, 0, (zFar + zNear) * d, 2 * d * zFar * zNear,
					         0, 0,                 -1,                    0
				);

			}
			
			mat4 ortho(T xLeft, T xRight, T yBottom, T yTop, T zNear, T zFar) {
				
				T tx, ty, tz;
				tx = - (xRight + xLeft) / (xRight - xLeft);
				ty = - (yTop + yBottom) / (yTop - yBottom);
				tz = - (zFar + zNear) / (zFar - zNear);				
				return mat4(				
					2 / (xRight - xLeft), 0,                  0,                tx,
					0,                  2 / (yTop - yBottom), 0,                ty,
					0,                  0,                 -2 / (zFar - zNear), tz,
					0,                  0,                  0,                1
				);
				
			}
			
			// use it to set up a camera (untested)
			mat4 lookAt(vec3!(T)eye, vec3!(T)center, vec3!(T)up)
			{
				auto f = (center - eye).normalized();
				auto upp = up.normalized();
				auto s = f.cross(upp);
				auto u = s.cross(f);
				
				
				auto rot = 
				mat4(				
					-s.x, u.x, -f.x, 0, 
					-s.y, u.y, -f.y, 0, 
					-s.z, u.z, -f.z, 0, 
					0  ,   0,    0, 1
				).transposed();
				
				auto tr = translate(-eye);
				
				return rot * tr;				
			}
			
		}		
		
	}
}

alias mat4!(float) mat4f;
alias mat4!(double) mat4d;