/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module util.geometry.vec2;

import util.math;
import util.geometry.vectorop;
import std.string;


struct vec2(T) 
{
	
	public {

		union {
			struct {
				T x, y;
			}
			T[2] v;
		}
	}
	
	mixin VectorOp!(T, v);
	
	
	public
	{		
		//////////////////
		// constructors //
		//////////////////
		
		static vec2 opCall(T x, T y)
		{
			vec2 v;
			v.x = x;
			v.y = y;
			return v;
		}
		
		static vec2 opCall(T x)
		{
			vec2 v;
			v.x = x;
			v.y = x;
			return v;
		}
		
		///////////////
		// constants //
		///////////////

		static vec2 ZEROS, ONES;
		
		static this()
		{
			ZEROS = vec2(0);
			ONES = vec2(1);
		}
		
		
		///////////////////////////
		// GLSL style properties //
		///////////////////////////
		
		vec2 xx() { return vec2(x, x); }
		vec2 xy() { return vec2(x, y); }
		vec2 yx() { return vec2(y, x); }
		vec2 yy() { return vec2(y, y); }
		
		
		//////////////////////////
		// operator overloading	//
		//////////////////////////
		
		vec2 opAdd(T a) 
		{
			return vec2(x + a, y + a);
		}

		vec2 opAdd(vec2 v) 
		{
			return vec2(x + v.x, y + v.y);
		}

		vec2 opSub(vec2 v) 
		{
			return vec2(x - v.x, y - v.y);
		}

		vec2 opSub(T a) 
		{
			return vec2(x - a, y - a);
		}

		vec2 opSub_r(T a) 
		{
			return vec2(a - x, a - y);
		}

		vec2 opMul(vec2 v) 
		{
			return vec2(x * v.x, y * v.y);
		}

		vec2 opMul(T a) 
		{
			return vec2(x * a, y * a);
		}
		
		vec2 opDiv(vec2 v) 
		{
			return vec2(x / v.x, y / v.y);
		}

		vec2 opDiv(T a) 
		{
			return vec2(x / a, y / a);
		}

		void opAddAssign(T a) 
		{
			x += a;
			y += a;
		}

		void opAddAssign(vec2 v) 
		{
			x += v.x;
			y += v.y;
		}

		void opSubAssign(T a) 
		{
			x -= a;
			y -= a;
		}

		void opSubAssign(vec2 v) 
		{
			x -= v.x;
			y -= v.y;
		}

		void opMulAssign(T a) 
		{
			x *= a;
			y *= a;
		}

		void opMulAssign(vec2 v) 
		{
			x *= v.x;
			y *= v.y;
		}

		void opDivAssign(T a) 
		{
			x /= a;
			y /= a;
		}

		void opDivAssign(vec2 v)
		{
			x /= v.x;
			y /= v.y;
		}

		vec2 opNeg() 
		{
			return vec2(-x, -y);
		}

		char[] toString() 
		{
			return format("(%s %s)", x, y);
		}
	}
}




alias vec2!(int) vec2i;
alias vec2!(float) vec2f;
alias vec2!(double) vec2d;

