/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module util.math;

version(D_InlineAsm_X86)
{
// OK	
} else
{
	static assert(0, "x86 support is mandatory.");
}

public import std.math;	
alias std.math.sinhf sinhf;
alias std.math.expf expf;
alias std.math.floorf floor;
alias std.math.floor floor;	


const double PI_D = 3.1415926535897932384626433832795;
const double TWO_PI_D = 2.0 * PI_D;
const double PI_DIV_2_D = 0.5 * PI_D;
const double PI_DIV_4_D = 0.25 * PI_D;

const float PI_F = PI_D;
const float TWO_PI_F = TWO_PI_D;
const float PI_DIV_2_F = PI_DIV_2_D;
const float PI_DIV_4_F = PI_DIV_4_D;

const float DEG_TO_RAD = PI_D / 180;
const float RAD_TO_DEG = 180 / PI_D;

const double SQRT_2_D = 1.4142135623730950488016887242097;
const float SQRT_2_F = SQRT_2_D;


// FPU constants
static final ushort FPU_ROUND_TO_NEAREST = 0x037F;
static final ushort FPU_ROUND_TOWARD_ZERO = 0x0F7F;

// min and max function
T min(S, T)(S a, T b)
{
	return (a < b) ? a : b;
}

T max(S, T)(S a, T b)
{
	return (a > b) ? a : b;
}

// specialization for int
T min(T:int)(int a, int b)
{
	asm
	{
		mov EDX, a;
		cmp EAX, EDX;
  		cmovg EAX, EDX;	
	}
}

T max(T:int)(int a, int b)
{
	asm
	{
		mov EDX, a;
		cmp EAX, EDX;
  		cmovl EAX, EDX;	
	}	
}


// specialization for float
T min(T:float)(float a, float b) 
{	
	asm
	{
		fld a;
		fld b;
		fcomi ST, ST(1);
		fcmovnb ST(1);
		ffree ST(1);
	}
}

T max(T:float)(float a, float b) 
{	
	asm
	{
	 	fld a;
    	fld b;
		fcomi ST, ST(1);
		fcmovb ST(1);
		ffree ST(1);
	}
}



T lerp(T, U)(T a, T b, U t)
{
	return t * b + ( cast(U)1 - t ) * a;
}

T step(S,T,U)(S a, T b, U t)
{
	if (t <= a) return 0.f;
	else if (t >= b) return 1.f;
	else return (t - a) / (b - a); 
}

T smoothStep(S,T,U)(S a, T b, U t)
{
	if (t <= a) return cast(U)0;
	else if (t >= b) return cast(U)1;
	else 
	{
		auto x = (t - a) / (b - a); 
		return x * x * (cast(U)3 - cast(U)2 * x); 		
	}
}

T clamp(T)(T x)
{
	return x > 1 ? 1 : (x < 0 ? 0 : x);	
}

/**
 * Fast clamping for float (between 0.f and 1.f)
 */
T clamp(T : float)(float x) 
{
	asm
	{
		mov EAX, x;
		mov ECX, 0;
		or EAX, EAX;
		cmovs EAX, ECX;
		mov ECX, 0x3F800000; // 1.f
		cmp EAX, ECX;
		cmova EAX, ECX;
		mov x, EAX;
		fld x;
	}
}

int round(T)(T x)
{
	asm
	{
		fldcw FPU_ROUND_TO_NEAREST;
		fld x;
		sub ESP, 4;
		fistp dword ptr [ESP];
		pop EAX;
	}
}


/* computes a sinus and cosinus at the same time */
void sincos(T)(T angle, inout T sin, inout T cos)
{
	T s, c;
	asm
	{
		fld	angle;
		fsincos;
		fstp s;
		fstp c;
	}
	sin = s;
	cos = c;
}

int trunc(T)(T x)
{
	asm
	{
		fldcw FPU_ROUND_TOWARD_ZERO;
		fld x;
		sub ESP, 4;
		fistp dword ptr [ESP];
		pop EAX;
	}
}

T frac(T)(T x)
{
	return x - cast(int)x;
}

T tan(T)(T x)
{
	/*version(D_InlineAsm_X86)
    {
	   	asm
	 	{
	 		fld x; 
	 		fptan; 
	 		fdecstp	;		
 	    }
	} else */
	{
		return cast(T)std.math.tan(x);
	}
}

T pow2(T)(T x)
{  
	
	asm
	{
		fld1;
		fld x;
		fprem;
		fstp st(1);
		f2xm1;
		fld1;
		fadd;
		fldcw FPU_ROUND_TOWARD_ZERO;
		fld [x];
		frndint;
		fld1;
		fscale;
		fstp st(1);
		fmul;
	}
}

T pow(T)(T base, T exponent)
{
	T tmp;
	asm
	{
		fld exponent;
  		fld base;
  		fyl2x;	
  		fst tmp;
  		fld1;
		fxch ST(1);
		fprem;
		fstp ST(1);
		f2xm1;
		fld1;
		faddp ST(1), ST;
		fldcw FPU_ROUND_TOWARD_ZERO;
		fld tmp;
		frndint;
		fld1;
		fscale;
		fstp ST(1);
		fmul;
	}
}

T log2(T)(T x)
{  
	asm
	{
		fld1;
		fld x;
		fyl2x;
	}
}	

T log10(T)(T x)
{  
	asm
	{
		fldlg2;
		fld x;
		fyl2x;
	}
}

T log(T)(T x)
{  
	asm
	{
		fldln2;
		fld x;
		fyl2x;
	}
}

/**
 * square 
 */
T sqr(T)(T s)
{
	return s * s;	
}

/**
 * square root
 */
T sqrt(T)(T x)
{
	asm
	{
		fld x;
		fsqrt;
	}	
}

/**
 * tangent function
 */
T atan(T)(T x)
{
	asm
	{
		fld x;
		fld1;
		fpatan;	
	}	
}

/**
 * returns the angle of the vector (x,y)
 * in [-PI, PI[
 */
T atan2(T)(T x, T y)
{
	T u;
	asm
	{
		fld x;
		fld y;
		fpatan;	
		fstp u;
	}
	return u;
}

/**
 * return the first m with m = 2^k and m >= n
 * n must be >= 0
 */
int nextPow2(int n)
{
	for (int i = 0; i < 31; i++)
	{
		int m = (1 << i);
		if ((1 << i) >= n) return m;
	}
	return -1;
}


bool isSubnormal(float f)
{
	uint *p = cast(uint *)&f;
	return ((*p & 0x7F800000) == 0) && (*p & 0x007FFFFF);
}

/**
 * linear interpolation
 */
T linearInterp(T)(T[] table, float index)
{
	if (index < 0)
	{
		return table[0];	
	}		
	else if (index >= table.length - 1) 
	{
		return table[table.length - 1];
	}
	else 
	{
		int ipart = cast(int)(index);
		float fpart = index - ipart;	
		T a = table[ipart] * (1 - fpart);	
		T b = table[ipart + 1] * fpart;	
		return a + b;
	}
}

